# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2015-2020 Rapptz
Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

This is a slightly modified version of discord.py's client
https://github.com/Rapptz/discord.py/blob/master/discord/client.py
"""

from __future__ import annotations

import asyncio
import datetime
import logging
import sys
import traceback
from typing import TYPE_CHECKING, Any, Callable, Coroutine, Optional, TypeVar, Union, overload

import aiohttp
from typing_extensions import Literal, final

from . import errors, utils
from .abc import SteamID
from .gateway import *
from .guard import generate_one_time_code
from .http import HTTPClient
from .iterators import TradesIterator
from .models import PriceOverview, community_route
from .state import ConnectionState

if TYPE_CHECKING:
    import steam

    from .abc import Message
    from .clan import Clan
    from .comment import Comment
    from .enums import EPersonaState, EPersonaStateFlag, EUIMode
    from .game import Game
    from .group import Group
    from .invite import ClanInvite, Invite, UserInvite
    from .protobufs import Msg, MsgProto
    from .trade import TradeOffer
    from .user import ClientUser, User

__all__ = ("Client",)

log = logging.getLogger(__name__)
TASK_HAS_NAME = sys.version_info[:2] >= (3, 8)
EventType = Callable[..., Coroutine[Any, Any, Any]]
E = TypeVar("E", bound=EventType)
EventDeco = Union[Callable[[E], E], E]


class Client:
    """Represents a client connection that connects to Steam. This class is used to interact with the Steam API and CMs.

    Parameters
    ----------
    game: :class:`~steam.Game`
        A games to set your status as on connect.
    games: list[:class:`~steam.Game`]
        A list of games to set your status to on connect.
    state: :class:`~steam.EPersonaState`
        The state to show your account as on connect.

        Note
        ----
        Setting your status to :attr:`~steam.EPersonaState.Offline`, will stop you receiving persona state updates
        and by extension :meth:`on_user_update` will stop being dispatched.

    ui_mode: :class:`~steam.EUIMode`
        The UI mode to set your status to on connect.
    force_kick: :class:`bool`
        Whether or not to forcefully kick any other playing sessions on connect.

    Attributes
    -----------
    ws:
        The connected websocket/CM server, this can be used to directly send messages to said CM.
    """

    def __init__(self, loop: Optional[asyncio.AbstractEventLoop] = None, **options: Any):
        if loop:
            utils.warn("The loop argument is deprecated and scheduled for removal in V.1")
        self.loop = asyncio.get_event_loop()
        self.http = HTTPClient(client=self, **options)
        self._connection = ConnectionState(client=self, **options)
        self.ws: Optional[SteamWebSocket] = None

        self.username: Optional[str] = None
        self.api_key: Optional[str] = None
        self.password: Optional[str] = None
        self.shared_secret: Optional[str] = None
        self.identity_secret: Optional[str] = None
        self.token: Optional[str] = None

        self._closed = True
        self._cm_list: Optional[CMServerList] = None
        self._listeners: dict[str, list[tuple[asyncio.Future, Callable[..., bool]]]] = {}
        self._ready = asyncio.Event()

    @property
    def user(self) -> Optional[ClientUser]:
        """Optional[:class:`~steam.ClientUser`]: Represents the connected client. ``None`` if not logged in."""
        return self.http.user

    @property
    def users(self) -> list[User]:
        """list[:class:`~steam.User`]: Returns a list of all the users the account can see."""
        return self._connection.users

    @property
    def trades(self) -> list[TradeOffer]:
        """list[:class:`~steam.TradeOffer`]: Returns a list of all the trades the connected client has seen."""
        return self._connection.trades

    @property
    def groups(self) -> list[Group]:
        """list[:class:`~steam.Group`]: Returns a list of all the groups the connected client is in."""
        return self._connection.groups

    @property
    def clans(self) -> list[Clan]:
        """list[:class:`~steam.Clan`]: Returns a list of all the clans the connected client is in."""
        return self._connection.clans

    @property
    def latency(self) -> float:
        """:class:`float`: Measures latency between a heartbeat send and the heartbeat interval in seconds."""
        return float("nan") if self.ws is None else self.ws.latency

    async def code(self) -> str:
        """|coro|

        Warning
        -------
        Will wait for a Steam guard code using :func:`input` in an executor if no shared_secret is passed to
        :meth:`run` or :meth:`start` blocking exiting until one is entered.

        Returns
        -------
        :class:`str`
            The current steam guard code.
        """
        if self.shared_secret:
            return generate_one_time_code(self.shared_secret)
        print("Please enter a Steam guard code")
        code = await utils.ainput(">>> ")
        return code.strip()

    def is_ready(self) -> bool:
        """:class:`bool`: Specifies if the client's internal cache is ready for use."""
        return self._ready.is_set()

    def is_closed(self) -> bool:
        """:class:`bool`: Indicates if connection is closed to the API or CMs."""
        return self._closed

    @overload
    def event(self, coro: Literal[None] = ...) -> Callable[[E], E]:
        ...

    @overload
    def event(self, coro: E) -> E:
        ...

    def event(self, coro: Optional[EventDeco] = None) -> EventDeco:
        """|maybecallabledeco|
        Register an event to listen to.

        The events must be a :ref:`coroutine <coroutine>`, if not, :exc:`TypeError` is raised.

        Usage: ::

            @client.event
            async def on_ready():
                print('Ready!')

        Raises
        ------
        :exc:`TypeError`
            The function passed is not a coroutine.
        """

        def decorator(coro: E) -> E:
            if not asyncio.iscoroutinefunction(coro):
                raise TypeError(f"Registered events must be a coroutines, {coro.__name__} is {type(coro).__name__}")

            setattr(self, coro.__name__, coro)
            log.debug(f"{coro.__name__} has been registered as an event")
            return coro

        return decorator(coro) if coro is not None else decorator

    async def _run_event(self, coro: EventType, event_name: str, *args: Any, **kwargs: Any) -> None:
        try:
            await coro(*args, **kwargs)
        except asyncio.CancelledError:
            pass
        except Exception as exc:
            try:
                await self.on_error(event_name, exc, *args, **kwargs)
            except asyncio.CancelledError:
                pass

    def _schedule_event(self, coro: EventType, event_name: str, *args: Any, **kwargs: Any) -> asyncio.Task:
        wrapped = self._run_event(coro, event_name, *args, **kwargs)
        return asyncio.Task(wrapped, name=event_name) if TASK_HAS_NAME else asyncio.Task(wrapped)

    def dispatch(self, event: str, *args: Any, **kwargs: Any) -> None:
        log.debug(f"Dispatching event {event}")
        method = f"on_{event}"

        listeners = self._listeners.get(event)
        # remove the dispatched listener
        if listeners:
            removed = []
            for idx, (future, condition) in enumerate(listeners):
                if future.cancelled():
                    removed.append(idx)
                    continue

                try:
                    result = condition(*args)
                except Exception as exc:
                    future.set_exception(exc)
                    removed.append(idx)
                else:
                    if result:
                        if len(args) == 0:
                            future.set_result(None)
                        elif len(args) == 1:
                            future.set_result(args[0])
                        else:
                            future.set_result(args)
                        removed.append(idx)

            if len(removed) == len(listeners):
                self._listeners.pop(event)
            else:
                for idx in reversed(removed):
                    del listeners[idx]

        # schedule the event (if possible)
        try:
            coro = getattr(self, method)
        except AttributeError:
            pass
        else:
            self._schedule_event(coro, method, *args, **kwargs)

    def _handle_ready(self) -> None:
        self._ready.set()
        self.dispatch("ready")

    @final
    def run(self, *args: Any, **kwargs: Any) -> None:
        """A blocking call that abstracts away the event loop initialisation from you.

        This is roughly equivalent to::

            try:
                asyncio.run(client.start(username, password))
            finally:
                asyncio.run(client.close())

        It is not recommended to subclass this method, it is normally favourable to subclass :meth:`start` as it is a
        :ref:`coroutine <coroutine>`.

        Note
        ----
        This takes the same arguments as :meth:`start`.
        """

        async def runner() -> None:
            asyncio.events.new_event_loop = old_new_event_loop
            try:
                await self.start(*args, **kwargs)
            finally:
                if not self.is_closed():
                    await self.close()

        # we just have to monkey patch in support for using get_event_loop
        old_new_event_loop = asyncio.new_event_loop
        asyncio.events.new_event_loop = asyncio.get_event_loop
        try:
            asyncio.run(runner())
        except KeyboardInterrupt:
            log.info("Closing the event loop")

    async def login(self, username: str, password: str, *, shared_secret: Optional[str] = None) -> None:
        """|coro|
        Logs in a Steam account and the Steam API with the specified credentials.

        Parameters
        ----------
        username: :class:`str`
            The username of the user's account.
        password: :class:`str`
            The password of the user's account.
        shared_secret: Optional[:class:`str`]
            The shared_secret of the desired Steam account, used to generate the 2FA code for login. If ``None`` is
            passed, the code will need to be inputted by the user via :meth:`code`.

        Raises
        ------
        :exc:`TypeError`
            Unexpected keyword arguments were received.
        :exc:`.InvalidCredentials`
            Invalid credentials were passed.
        :exc:`.HTTPException`
            An unknown HTTP related error occurred.
        :exc:`.NoCMsFound`
            No community managers could be found to connect to.
        """
        log.info(f"Logging in to steamcommunity.com")
        self.username = username
        self.password = password
        self.shared_secret = shared_secret

        await self.http.login(username, password, shared_secret=shared_secret)
        self._closed = False
        self.loop.create_task(self._connection.__ainit__())

    async def close(self) -> None:
        """|coro|
        Closes the connection to Steam.
        """
        if self.is_closed():
            return

        self._closed = True
        await self.http.close()

        if self.ws is not None:
            try:
                await self.ws.handle_close()
            except ConnectionClosed:
                pass

        self._ready.clear()

    def clear(self) -> None:
        """Clears the internal state of the bot. After this, the bot can be considered "re-opened", i.e.
        :meth:`is_closed` and :meth:`is_ready` both return ``False``. This also clears the internal cache.
        """
        self._closed = False
        self._ready.clear()
        self._connection.clear()
        self.http.recreate()

    async def start(
        self,
        username: str,
        password: str,
        *,
        shared_secret: Optional[str] = None,
        identity_secret: Optional[str] = None,
    ) -> None:
        """|coro|
        A shorthand coroutine for :meth:`login` and :meth:`connect`. If no ``shared_secret`` is passed, you will have to
        manually enter a Steam guard code using :meth:`code`.

        Parameters
        -----------
        username: :class:`str`
            The username of the account to login to.
        password: :class:`str`
            The password of the account to login to.
        shared_secret: Optional[:class:`str`]
            The shared secret for the account to login to.
        identity_secret: Optional[:class:`str`]
            The identity secret for the account to login to.

        Raises
        ------
        :exc:`.InvalidCredentials`
            The wrong credentials are passed.
        :exc:`.HTTPException`
            An unknown HTTP related error occurred.
        :exc:`.NoCMsFound`
            No community managers could be found to connect to.
        """

        self.username = username
        self.password = password
        self.shared_secret = shared_secret
        self.identity_secret = identity_secret

        if identity_secret is None:
            log.info("Trades will not be automatically accepted when sent as no identity_secret was passed.")

        await self.login(username, password, shared_secret=shared_secret)
        await self.connect()

    async def connect(self) -> None:
        """|coro|
        Initialize a connection to a Steam CM after logging in.
        """
        while not self.is_closed():
            try:
                resp = await self.http.request("GET", url=community_route("chat/clientjstoken"))
                if not resp["logged_in"]:  # we got logged out :(
                    await self.http.login(self.username, self.password, shared_secret=self.shared_secret)
                    continue
                self.token = resp["token"]
                coro = SteamWebSocket.from_client(self, cm_list=self._cm_list)
                self.ws: SteamWebSocket = await asyncio.wait_for(coro, timeout=60)
                while True:
                    await self.ws.poll_event()
            except (
                OSError,
                ConnectionClosed,
                aiohttp.ClientError,
                ConnectionResetError,
                asyncio.TimeoutError,
                errors.HTTPException,
            ) as exc:
                if isinstance(exc, ConnectionClosed):
                    self._cm_list = exc.cm_list
                self.dispatch("disconnect")
            finally:
                if not self.is_closed():
                    log.info(f"Attempting to connect to another CM")
                    await asyncio.sleep(5)

    # state stuff

    def get_user(self, id: utils.Intable) -> Optional[User]:
        """Returns a user from cache with a matching ID.

        Parameters
        ----------
        id: Union[:class:`int`, :class:`str`]
            The ID of the user, can be an :attr:`.SteamID.id64`, :attr:`.SteamID.id`, :attr:`.SteamID.id2` or an
            :attr:`.SteamID.id3`.

        Returns
        -------
        Optional[:class:`~steam.User`]
            The user or ``None`` if the user was not found.
        """
        steam_id = SteamID(id=id, type="Individual")
        return self._connection.get_user(steam_id.id64)

    async def fetch_user(self, id: utils.Intable) -> Optional[User]:
        """|coro|
        Fetches a user from the API with a matching ID.

        Parameters
        ----------
        id: Union[:class:`int`, :class:`str`]
            The ID of the user, can be an :attr:`.SteamID.id64`, :attr:`.SteamID.id`, :attr:`.SteamID.id2` or an
            :attr:`.SteamID.id3`.

        Returns
        -------
        Optional[:class:`~steam.User`]
            The user or ``None`` if the user was not found.
        """
        steam_id = SteamID(id=id, type="Individual")
        return await self._connection.fetch_user(steam_id.id64)

    async def fetch_users(self, *ids: utils.Intable) -> list[Optional[User]]:
        """|coro|
        Fetches a list of :class:`~steam.User` from their IDs from the API with a matching ID. The
        :class:`~steam.User` objects returned are unlikely to retain the order they were originally in.

        Parameters
        ----------
        *ids: Union[:class:`int`, :class:`str`]
            The user's IDs.

        Returns
        -------
        list[Optional[:class:`~steam.User`]]
            A list of the users or ``None`` if the user was not found.
        """
        steam_ids = [SteamID(id).id64 for id in ids]
        return await self._connection.fetch_users(steam_ids)

    async def fetch_user_named(self, name: str) -> Optional[User]:
        """|coro|
        Fetches a user from https://steamcommunity.com from there community URL name.

        Parameters
        ----------
        name: :class:`str`
            The name of the user after https://steamcommunity.com/id

        Returns
        -------
        Optional[:class:`~steam.User`]
            The user or ``None`` if the user was not found.
        """
        id64 = await utils.id64_from_url(community_route(f"id/{name}"), self.http._session)
        return await self._connection.fetch_user(id64) if id64 is not None else None

    def get_trade(self, id: int) -> Optional[TradeOffer]:
        """Get a trade from cache with a matching ID.

        Parameters
        ----------
        id: :class:`int`
            The id of the trade to search for from the cache.

        Returns
        -------
        Optional[:class:`~steam.TradeOffer`]
            The trade offer or ``None`` if the trade was not found.
        """
        return self._connection.get_trade(id)

    async def fetch_trade(self, id: int) -> Optional[TradeOffer]:
        """|coro|
        Fetches a trade from the API with a matching ID.

        Parameters
        ----------
        id: :class:`int`
            The ID of the trade to search for from the API.

        Returns
        -------
        Optional[:class:`~steam.TradeOffer`]
            The trade offer or ``None`` if the trade was not found.
        """
        return await self._connection.fetch_trade(id)

    def get_group(self, id: utils.Intable) -> Optional[Group]:
        """Get a group from cache with a matching ID.

        Parameters
        ----------
        id: Union[:class:`int`, :class:`str`]
            The ID of the group, can be an :attr:`.SteamID.id64`, :attr:`.SteamID.id`, :attr:`.SteamID.id2` or an
            :attr:`.SteamID.id3`.

        Returns
        -------
        Optional[:class:`~steam.Group`]
            The group or ``None`` if the group was not found.
        """
        steam_id = SteamID(id=id, type="Chat")
        return self._connection.get_group(steam_id.id)

    def get_clan(self, id: utils.Intable) -> Optional[Clan]:
        """Get a clan from cache with a matching ID.

        Parameters
        ----------
        id: Union[:class:`int`, :class:`str`]
            The ID of the clan, can be an :attr:`.SteamID.id64`, :attr:`.SteamID.id`, :attr:`.SteamID.id2` or an
            :attr:`.SteamID.id3`.

        Returns
        -------
        Optional[:class:`~steam.Clan`]
            The clan or ``None`` if the clan was not found.
        """
        steam_id = SteamID(id=id, type="Clan")
        return self._connection.get_clan(steam_id.id)

    async def fetch_clan(self, id: utils.Intable) -> Optional[Clan]:
        """|coro|
        Fetches a clan from the websocket with a matching ID.

        Parameters
        ----------
        id: Union[:class:`int`, :class:`str`]
            The ID of the clan, can be an :attr:`.SteamID.id64`, :attr:`.SteamID.id`, :attr:`.SteamID.id2` or an
            :attr:`.SteamID.id3`.


        Returns
        -------
        Optional[:class:`~steam.Clan`]
            The clan or ``None`` if the clan was not found.
        """
        steam_id = SteamID(id=id, type="Clan")
        return await self._connection.fetch_clan(steam_id.id64)

    async def fetch_clan_named(self, name: str) -> Optional[Clan]:
        """|coro|
        Fetches a clan from https://steamcommunity.com with a matching name.

        Parameters
        ----------
        name: :class:`str`
            The name of the Steam clan.

        Returns
        -------
        Optional[:class:`~steam.Clan`]
            The clan or ``None`` if the clan was not found.
        """
        steam_id = await SteamID.from_url(community_route(f"clans/{name}"), self.http._session)
        if steam_id is None:
            return None
        return await self._connection.fetch_clan(steam_id.id64)

    def trade_history(
        self,
        limit: Optional[int] = 100,
        before: Optional[datetime.datetime] = None,
        after: Optional[datetime.datetime] = None,
        active_only: bool = False,
    ) -> TradesIterator:
        """An :class:`~steam.iterators.AsyncIterator` for accessing a :class:`steam.ClientUser`'s
        :class:`steam.TradeOffer` objects.

        Examples
        -----------

        Usage: ::

            async for trade in client.trade_history(limit=10):
                print('Partner:', trade.partner, 'Sent:')
                print(', '.join(item.name if item.name else str(item.asset_id) for item in trade.items_to_receive)
                      if trade.items_to_receive else 'Nothing')

        Flattening into a list: ::

            trades = await client.trade_history(limit=50).flatten()
            # trades is now a list of TradeOffer

        All parameters are optional.

        Parameters
        ----------
        limit: Optional[:class:`int`]
            The maximum number of trades to search through. Default is 100 which will fetch the first 100 trades.
            Setting this to ``None`` will fetch all of the user's trades, but this will be a very slow operation.
        before: Optional[:class:`datetime.datetime`]
            A time to search for trades before.
        after: Optional[:class:`datetime.datetime`]
            A time to search for trades after.
        active_only: Optional[:class:`bool`]
            Whether or not to fetch only active trades defaults to ``True``.

        Yields
        ---------
        :class:`~steam.TradeOffer`
        """
        return TradesIterator(state=self._connection, limit=limit, before=before, after=after, active_only=active_only)

    # miscellaneous stuff

    async def change_presence(
        self,
        *,
        game: Optional[Game] = None,
        games: Optional[list[Game]] = None,
        state: Optional[EPersonaState] = None,
        ui_mode: Optional[EUIMode] = None,
        flag: Optional[EPersonaStateFlag] = None,
        flags: Optional[list[EPersonaStateFlag]] = None,
        force_kick: bool = False,
    ) -> None:
        """|coro|
        Set your status.

        Parameters
        ----------
        game: Optional[:class:`~steam.Game`]
            A games to set your status as.
        games: Optional[list[:class:`~steam.Game`]]
            A list of games to set your status to.
        state: Optional[:class:`~steam.EPersonaState`]
            The state to show your account as.

            Warning
            -------
            Setting your status to :attr:`~steam.EPersonaState.Offline`, will stop you receiving persona state updates
            and by extension :meth:`on_user_update` will stop being dispatched.

        ui_mode: Optional[:class:`~steam.EUIMode`]
            The UI mode to set your status to.
        flag: Optional[:class:`EPersonaStateFlag`]
            The flag to update your account with.
        flags: Optional[list[:class:`EPersonaStateFlag`]
            The flags to update your account with.
        force_kick: :class:`bool`
            Whether or not to forcefully kick any other playing sessions.
        """
        games = [game.to_dict() for game in games] if games is not None else []
        if game is not None:
            games.append(game.to_dict())
        flags = flags or self.user.flags
        if flag is not None:
            flags.append(flag)
        flag_value = 0
        for flag in flags:
            flag_value |= flag
        await self.ws.change_presence(
            games=games, state=state, flags=flag_value, ui_mode=ui_mode, force_kick=force_kick
        )

    async def wait_until_ready(self) -> None:
        """|coro|
        Waits until the client's internal cache is all ready.
        """
        await self._ready.wait()

    async def fetch_price(self, name: str, game: Game, currency: Optional[int] = None) -> PriceOverview:
        """|coro|
        Fetch the price for an item.

        Parameters
        ----------
        name: :class:`str`
            The name of the item.
        game: :class:`.Game`
            The game the item is from.
        currency: :class:`int`
            The currency to fetch the price in.

        Returns
        --------
        :class:`.PriceOverview`
            The price overview for the item.
        """
        price = await self.http.get_price(game.id, name, currency)
        return PriceOverview(price)

    # events to be subclassed

    async def on_error(self, event: str, error: Exception, *args, **kwargs):
        """|coro|
        The default error handler provided by the client.

        Usually when an event raises an uncaught exception, a traceback is printed to :attr:`sys.stderr` and the
        exception is ignored. If you want to change this behaviour and handle the exception yourself, this event can
        be overridden. Which, when done, will suppress the default action of printing the traceback.

        If you want exception to propagate out of the :class:`Client` class you can define an ``on_error`` handler
        consisting of a single empty :ref:`py:raise`. Exceptions raised by ``on_error`` will not be handled in any
        way by :class:`Client`.

        Parameters
        ----------
        event: :class:`str`
            The name of the event that errored.
        error: :exc:`Exception`
            The error that was raised.
        *args:
            The positional arguments associated with the event.
        **kwargs:
            The key-word arguments associated with the event.
        """
        print(f"Ignoring exception in {event}", file=sys.stderr)
        traceback.print_exception(type(error), error, error.__traceback__, file=sys.stderr)

    if TYPE_CHECKING:  # these methods shouldn't exist at runtime unless subclassed to prevent pollution of logs

        async def on_connect(self) -> None:
            """|coro|
            Called when the client has successfully connected to Steam. This is not the same as the client being
            fully prepared, see :func:`on_ready` for that.

            The warnings on :meth:`on_ready` also apply.
            """

        async def on_disconnect(self) -> None:
            """|coro|
            Called when the client has disconnected from Steam. This could happen either through the internet
            disconnecting, an explicit call to logout, or Steam terminating the connection.

            This function can be called multiple times.
            """

        async def on_ready(self) -> None:
            """|coro|
            Called after a successful login and the client has handled setting up everything.

            Warning
            -------
            This function is not guaranteed to be the first event called. Likewise, this function is **not**
            guaranteed to only be called once. This library implements reconnection logic and will therefore
            end up calling this event whenever a CM disconnects.
            """

        async def on_login(self) -> None:
            """|coro|
            Called when the client has logged into https://steamcommunity.com.
            """

        async def on_logout(self) -> None:
            """|coro|
            Called when the client has logged out of https://steamcommunity.com.
            """

        async def on_message(self, message: "steam.Message") -> None:
            """|coro|
            Called when a message is created.

            Parameters
            ----------
            message: :class:`~steam.Message`
                The message that was received.
            """

        async def on_typing(self, user: "steam.User", when: "datetime.datetime") -> None:
            """|coro|
            Called when typing is started.

            Parameters
            ----------
            user: :class:`~steam.User`
                The user that started typing.
            when: :class:`datetime.datetime`
                The time the user started typing at.
            """

        async def on_trade_receive(self, trade: "steam.TradeOffer") -> None:
            """|coro|
            Called when the client receives a trade offer.

            Parameters
            ----------
            trade: :class:`~steam.TradeOffer`
                The trade offer that was received.
            """

        async def on_trade_send(self, trade: "steam.TradeOffer") -> None:
            """|coro|
            Called when the client sends a trade offer.

            Parameters
            ----------
            trade: :class:`~steam.TradeOffer`
                The trade offer that was sent.
            """

        async def on_trade_accept(self, trade: "steam.TradeOffer") -> None:
            """|coro|
            Called when the client or the trade partner accepts a trade offer.

            Parameters
            ----------
            trade: :class:`~steam.TradeOffer`
                The trade offer that was accepted.
            """

        async def on_trade_decline(self, trade: "steam.TradeOffer") -> None:
            """|coro|
            Called when the client or the trade partner declines a trade offer.

            Parameters
            ----------
            trade: :class:`~steam.TradeOffer`
                The trade offer that was declined.
            """

        async def on_trade_cancel(self, trade: "steam.TradeOffer") -> None:
            """|coro|
            Called when the client or the trade partner cancels a trade offer.

            Note
            ----
            This is called when the trade state becomes :attr:`~steam.ETradeOfferState.Canceled` and
            :attr:`~steam.ETradeOfferState.CanceledBySecondaryFactor`.

            Parameters
            ----------
            trade: :class:`~steam.TradeOffer`
                The trade offer that was cancelled.
            """

        async def on_trade_expire(self, trade: "steam.TradeOffer") -> None:
            """|coro|
            Called when a trade offer expires due to being active for too long.

            Parameters
            ----------
            trade: :class:`~steam.TradeOffer`
                The trade offer that expired.
            """

        async def on_trade_counter(self, trade: "steam.TradeOffer") -> None:
            """|coro|
            Called when the client or the trade partner counters a trade offer.

            Parameters
            ----------
            trade: :class:`~steam.TradeOffer`
                The trade offer that was countered.
            """

        async def on_comment(self, comment: "steam.Comment") -> None:
            """|coro|
            Called when the client receives a comment notification.

            Parameters
            ----------
            comment: :class:`~steam.Comment`
                The comment received.
            """

        async def on_user_invite(self, invite: "steam.UserInvite") -> None:
            """|coro|
            Called when the client receives/sends an invite from/to a :class:`~steam.User` to become a friend.

            Parameters
            ----------
            invite: :class:`~steam.UserInvite`
                The invite received.
            """

        async def on_user_invite_accept(self, invite: "steam.UserInvite") -> None:
            """|coro|
            Called when the client/invitee accepts an invite from/to a :class:`~steam.User` to become a friend.

            Parameters
            ----------
            invite: :class:`~steam.UserInvite`
                The invite that was accepted.
            """

        async def on_clan_invite(self, invite: "steam.ClanInvite") -> None:
            """|coro|
            Called when the client receives/sends an invite from/to a :class:`~steam.User` to join a
            :class:`~steam.Clan`.

            Parameters
            ----------
            invite: :class:`~steam.ClanInvite`
                The invite received.
            """

        async def on_clan_invite_accept(self, invite: "steam.ClanInvite") -> None:
            """|coro|
            Called when the client/invitee accepts an invite to join a :class:`~steam.Clan`.

            Parameters
            ----------
            invite: :class:`~steam.ClanInvite`
                The invite that was accepted.
            """

        async def on_user_update(self, before: "steam.User", after: "steam.User") -> None:
            """|coro|
            Called when a user's their state, due to one or more of the following attributes changing:

                - :attr:`~steam.User.name`
                - :attr:`~steam.User.state`
                - :attr:`~steam.User.flags`
                - :attr:`~steam.User.avatar_url`
                - :attr:`~steam.User.last_logon`
                - :attr:`~steam.User.last_logoff`
                - :attr:`~steam.User.last_seen_online`
                - :attr:`~steam.User.game`


            Parameters
            ----------
            before: :class:`~steam.User`
                The user's state before it was updated.
            after: :class:`~steam.User`
                The user's state now.
            """

        async def on_socket_receive(self, msg: "Union[Msg, MsgProto]") -> None:
            """|coro|
            Called when the connected web-socket parses a received ``Msg``/``MsgProto``

            Parameters
            ----------
            msg: Union[:class:`~steam.protobufs.Msg`, :class:`~steam.protobufs.MsgProto`]
                The received message.
            """

        async def on_socket_raw_receive(self, message: bytes) -> None:
            """|coro|
            Called when the connected web-socket receives raw :class:`bytes`. This isn't likely to be very useful.

            Parameters
            ----------
            message: bytes
                The raw received message.
            """

        async def on_socket_send(self, msg: "Union[Msg, MsgProto]") -> None:
            """|coro|
            Called when the client sends a parsed ``Msg``/``MsgProto`` to the connected web-socket.

            Parameters
            ----------
            msg: Union[:class:`~steam.protobufs.Msg`, :class:`~steam.protobufs.MsgProto`]
                The sent message.
            """

        async def on_socket_raw_send(self, message: bytes) -> None:
            """|coro|
            Called when the client sends raw :class:`bytes` to the connected web-socket. This isn't likely to be very
            useful.

            Parameters
            ----------
            message: bytes
                The raw sent message.
            """

    @overload
    async def wait_for(
        self,
        event: Literal[
            "connect",
            "disconnect",
            "ready",
            "login",
            "logout",
        ],
        *,
        check: Optional[Callable[[], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> None:
        ...

    @overload
    async def wait_for(
        self,
        event: Literal["error"],
        *,
        check: Optional[Callable[[str, Exception, tuple[Any, ...], dict[str, Any]], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> tuple[str, Exception, tuple, dict]:
        ...

    @overload
    async def wait_for(
        self,
        event: Literal["message"],
        *,
        check: Optional[Callable[[Message], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> Message:
        ...

    @overload
    async def wait_for(
        self,
        event: Literal["comment"],
        *,
        check: Optional[Callable[[Comment], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> Comment:
        ...

    @overload
    async def wait_for(
        self,
        event: Literal["user_update"],
        *,
        check: Optional[Callable[[User, User], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> tuple[User, User]:
        ...

    @overload
    async def wait_for(
        self,
        event: Literal["typing"],
        *,
        check: Optional[Callable[[User, datetime.datetime], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> tuple[User, datetime.datetime]:
        ...

    @overload
    async def wait_for(
        self,
        event: Literal[
            "trade_receive",
            "trade_send",
            "trade_accept",
            "trade_decline",
            "trade_cancel",
            "trade_expire",
            "trade_counter",
        ],
        *,
        check: Optional[Callable[[TradeOffer], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> TradeOffer:
        ...

    @overload
    async def wait_for(
        self,
        event: Literal[
            "user_invite",
            "user_invite_accept",
        ],
        *,
        check: Optional[Callable[[UserInvite], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> UserInvite:
        ...

    @overload
    async def wait_for(
        self,
        event: Literal[
            "clan_invite",
            "clan_invite_accept",
        ],
        *,
        check: Optional[Callable[[ClanInvite], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> ClanInvite:
        ...

    @overload
    async def wait_for(
        self,
        event: Literal[
            "socket_receive",
            "socket_send",
        ],
        *,
        check: Optional[Callable[[Msgs], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> Msgs:
        ...

    @overload
    async def wait_for(
        self,
        event: Literal[
            "socket_raw_receive",
            "socket_raw_send",
        ],
        *,
        check: Optional[Callable[[bytes], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> bytes:
        ...

    async def wait_for(
        self,
        event: str,
        *,
        check: Optional[Callable[..., bool]] = None,
        timeout: Optional[float] = None,
    ) -> Any:
        """|coro|
        Wait for the first event to be dispatched that meets the requirements, this by default is the first event
        with a matching event name.

        Parameters
        -----------
        event: :class:`str`
            The event name from the :ref:`event reference <event-reference>`, but without the ``on_`` prefix, to wait
            for.
        check: Optional[Callable[..., :class:`bool`]]
            A callable predicate that checks the received event. The arguments must match the parameters of the
            ``event`` being waited for and must return a :class:`bool`.
        timeout: Optional[:class:`float`]
            By default, :meth:`wait_for` function does not timeout, however, in the case a ``timeout`` parameter is
            passed after the amount of seconds pass :exc:`asyncio.TimeoutError` is raised.

        Raises
        -------
        :exc:`asyncio.TimeoutError`
            If the provided timeout was reached.

        Returns
        --------
        Optional[Any]
            Returns ``None``, a single argument, or a :class:`tuple` of multiple arguments that mirrors the parameters
            for the ``event`` parameter from the :ref:`event reference <event-reference>`.
        """
        future = self.loop.create_future()
        check = check or return_true

        event_lower = event.lower()
        try:
            listeners = self._listeners[event_lower]
        except KeyError:
            listeners = []
            self._listeners[event_lower] = listeners

        listeners.append((future, check))
        return await asyncio.wait_for(future, timeout)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_clientserver.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto

from .steammessages_base import CMsgAuthTicket, CMsgIpAddress


class EMMSLobbyStatus(betterproto.Enum):
    Invalid = 0
    Exists = 1
    DoesNotExist = 2
    NotAMember = 3


@dataclass(eq=False, repr=False)
class CMsgClientUDSP2PSessionStarted(betterproto.Message):
    steamid_remote: float = betterproto.fixed64_field(1)
    appid: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUDSP2PSessionEnded(betterproto.Message):
    steamid_remote: float = betterproto.fixed64_field(1)
    appid: int = betterproto.int32_field(2)
    session_length_sec: int = betterproto.int32_field(3)
    session_error: int = betterproto.int32_field(4)
    nattype: int = betterproto.int32_field(5)
    bytes_recv: int = betterproto.int32_field(6)
    bytes_sent: int = betterproto.int32_field(7)
    bytes_sent_relay: int = betterproto.int32_field(8)
    bytes_recv_relay: int = betterproto.int32_field(9)
    time_to_connect_ms: int = betterproto.int32_field(10)


@dataclass(eq=False, repr=False)
class CMsgClientRegisterAuthTicketWithCM(betterproto.Message):
    protocol_version: int = betterproto.uint32_field(1)
    ticket: bytes = betterproto.bytes_field(3)
    client_instance_id: int = betterproto.uint64_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientTicketAuthComplete(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)
    game_id: float = betterproto.fixed64_field(2)
    estate: int = betterproto.uint32_field(3)
    eauth_session_response: int = betterproto.uint32_field(4)
    deprecated_ticket: bytes = betterproto.bytes_field(5)
    ticket_crc: int = betterproto.uint32_field(6)
    ticket_sequence: int = betterproto.uint32_field(7)
    owner_steam_id: float = betterproto.fixed64_field(8)


@dataclass(eq=False, repr=False)
class CMsgClientCMList(betterproto.Message):
    cm_addresses: List[int] = betterproto.uint32_field(1)
    cm_ports: List[int] = betterproto.uint32_field(2)
    cm_websocket_addresses: List[str] = betterproto.string_field(3)
    percent_default_to_websocket: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientP2PConnectionInfo(betterproto.Message):
    steam_id_dest: float = betterproto.fixed64_field(1)
    steam_id_src: float = betterproto.fixed64_field(2)
    app_id: int = betterproto.uint32_field(3)
    candidate: bytes = betterproto.bytes_field(4)
    connection_id_src: float = betterproto.fixed64_field(5)
    rendezvous: bytes = betterproto.bytes_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientP2PConnectionFailInfo(betterproto.Message):
    steam_id_dest: float = betterproto.fixed64_field(1)
    steam_id_src: float = betterproto.fixed64_field(2)
    app_id: int = betterproto.uint32_field(3)
    ep2p_session_error: int = betterproto.uint32_field(4)
    connection_id_dest: float = betterproto.fixed64_field(5)
    close_reason: int = betterproto.uint32_field(7)
    close_message: str = betterproto.string_field(8)


@dataclass(eq=False, repr=False)
class CMsgClientNetworkingCertRequest(betterproto.Message):
    key_data: bytes = betterproto.bytes_field(2)
    app_id: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientNetworkingCertReply(betterproto.Message):
    cert: bytes = betterproto.bytes_field(4)
    ca_key_id: float = betterproto.fixed64_field(5)
    ca_signature: bytes = betterproto.bytes_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientNetworkingMobileCertRequest(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientNetworkingMobileCertReply(betterproto.Message):
    encoded_cert: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientGetAppOwnershipTicket(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientGetAppOwnershipTicketResponse(betterproto.Message):
    eresult: int = betterproto.uint32_field(1)
    app_id: int = betterproto.uint32_field(2)
    ticket: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientSessionToken(betterproto.Message):
    token: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientGameConnectTokens(betterproto.Message):
    max_tokens_to_keep: int = betterproto.uint32_field(1)
    tokens: List[bytes] = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class CMsgGSServerType(betterproto.Message):
    app_id_served: int = betterproto.uint32_field(1)
    flags: int = betterproto.uint32_field(2)
    deprecated_game_ip_address: int = betterproto.uint32_field(3)
    game_port: int = betterproto.uint32_field(4)
    game_dir: str = betterproto.string_field(5)
    game_version: str = betterproto.string_field(6)
    game_query_port: int = betterproto.uint32_field(7)


@dataclass(eq=False, repr=False)
class CMsgGSStatusReply(betterproto.Message):
    is_secure: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class CMsgGSPlayerList(betterproto.Message):
    players: List["CMsgGSPlayerListPlayer"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CMsgGSPlayerListPlayer(betterproto.Message):
    steam_id: int = betterproto.uint64_field(1)
    deprecated_public_ip: int = betterproto.uint32_field(2)
    token: bytes = betterproto.bytes_field(3)
    public_ip: "CMsgIpAddress" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class CMsgGSUserPlaying(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)
    deprecated_public_ip: int = betterproto.uint32_field(2)
    token: bytes = betterproto.bytes_field(3)
    public_ip: "CMsgIpAddress" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class CMsgGSDisconnectNotice(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientGamesPlayed(betterproto.Message):
    games_played: List["CMsgClientGamesPlayedGamePlayed"] = betterproto.message_field(1)
    client_os_type: int = betterproto.uint32_field(2)
    cloud_gaming_platform: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientGamesPlayedGamePlayed(betterproto.Message):
    steam_id_gs: int = betterproto.uint64_field(1)
    game_id: float = betterproto.fixed64_field(2)
    deprecated_game_ip_address: int = betterproto.uint32_field(3)
    game_port: int = betterproto.uint32_field(4)
    is_secure: bool = betterproto.bool_field(5)
    token: bytes = betterproto.bytes_field(6)
    game_extra_info: str = betterproto.string_field(7)
    game_data_blob: bytes = betterproto.bytes_field(8)
    process_id: int = betterproto.uint32_field(9)
    streaming_provider_id: int = betterproto.uint32_field(10)
    game_flags: int = betterproto.uint32_field(11)
    owner_id: int = betterproto.uint32_field(12)
    vr_hmd_vendor: str = betterproto.string_field(13)
    vr_hmd_model: str = betterproto.string_field(14)
    launch_option_type: int = betterproto.uint32_field(15)
    primary_controller_type: int = betterproto.int32_field(16)
    primary_steam_controller_serial: str = betterproto.string_field(17)
    total_steam_controller_count: int = betterproto.uint32_field(18)
    total_non_steam_controller_count: int = betterproto.uint32_field(19)
    controller_workshop_file_id: int = betterproto.uint64_field(20)
    launch_source: int = betterproto.uint32_field(21)
    vr_hmd_runtime: int = betterproto.uint32_field(22)
    game_ip_address: "CMsgIpAddress" = betterproto.message_field(23)
    controller_connection_type: int = betterproto.uint32_field(24)


@dataclass(eq=False, repr=False)
class CMsgGSApprove(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)
    owner_steam_id: float = betterproto.fixed64_field(2)


@dataclass(eq=False, repr=False)
class CMsgGSDeny(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)
    edeny_reason: int = betterproto.int32_field(2)
    deny_string: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CMsgGSKick(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)
    edeny_reason: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientAuthList(betterproto.Message):
    tokens_left: int = betterproto.uint32_field(1)
    last_request_seq: int = betterproto.uint32_field(2)
    last_request_seq_from_server: int = betterproto.uint32_field(3)
    tickets: List["CMsgAuthTicket"] = betterproto.message_field(4)
    app_ids: List[int] = betterproto.uint32_field(5)
    message_sequence: int = betterproto.uint32_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientAuthListAck(betterproto.Message):
    ticket_crc: List[int] = betterproto.uint32_field(1)
    app_ids: List[int] = betterproto.uint32_field(2)
    message_sequence: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientLicenseList(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    licenses: List["CMsgClientLicenseListLicense"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientLicenseListLicense(betterproto.Message):
    package_id: int = betterproto.uint32_field(1)
    time_created: float = betterproto.fixed32_field(2)
    time_next_process: float = betterproto.fixed32_field(3)
    minute_limit: int = betterproto.int32_field(4)
    minutes_used: int = betterproto.int32_field(5)
    payment_method: int = betterproto.uint32_field(6)
    flags: int = betterproto.uint32_field(7)
    purchase_country_code: str = betterproto.string_field(8)
    license_type: int = betterproto.uint32_field(9)
    territory_code: int = betterproto.int32_field(10)
    change_number: int = betterproto.int32_field(11)
    owner_id: int = betterproto.uint32_field(12)
    initial_period: int = betterproto.uint32_field(13)
    initial_time_unit: int = betterproto.uint32_field(14)
    renewal_period: int = betterproto.uint32_field(15)
    renewal_time_unit: int = betterproto.uint32_field(16)
    access_token: int = betterproto.uint64_field(17)


@dataclass(eq=False, repr=False)
class CMsgClientLBSSetScore(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    leaderboard_id: int = betterproto.int32_field(2)
    score: int = betterproto.int32_field(3)
    details: bytes = betterproto.bytes_field(4)
    upload_score_method: int = betterproto.int32_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientLBSSetScoreResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    leaderboard_entry_count: int = betterproto.int32_field(2)
    score_changed: bool = betterproto.bool_field(3)
    global_rank_previous: int = betterproto.int32_field(4)
    global_rank_new: int = betterproto.int32_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientLBSSetUGC(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    leaderboard_id: int = betterproto.int32_field(2)
    ugc_id: float = betterproto.fixed64_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientLBSSetUGCResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientLBSFindOrCreateLB(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    leaderboard_sort_method: int = betterproto.int32_field(2)
    leaderboard_display_type: int = betterproto.int32_field(3)
    create_if_not_found: bool = betterproto.bool_field(4)
    leaderboard_name: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientLBSFindOrCreateLBResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    leaderboard_id: int = betterproto.int32_field(2)
    leaderboard_entry_count: int = betterproto.int32_field(3)
    leaderboard_sort_method: int = betterproto.int32_field(4)
    leaderboard_display_type: int = betterproto.int32_field(5)
    leaderboard_name: str = betterproto.string_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientLBSGetLBEntries(betterproto.Message):
    app_id: int = betterproto.int32_field(1)
    leaderboard_id: int = betterproto.int32_field(2)
    range_start: int = betterproto.int32_field(3)
    range_end: int = betterproto.int32_field(4)
    leaderboard_data_request: int = betterproto.int32_field(5)
    steamids: List[float] = betterproto.fixed64_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientLBSGetLBEntriesResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    leaderboard_entry_count: int = betterproto.int32_field(2)
    entries: List["CMsgClientLBSGetLBEntriesResponseEntry"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientLBSGetLBEntriesResponseEntry(betterproto.Message):
    steam_id_user: float = betterproto.fixed64_field(1)
    global_rank: int = betterproto.int32_field(2)
    score: int = betterproto.int32_field(3)
    details: bytes = betterproto.bytes_field(4)
    ugc_id: float = betterproto.fixed64_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientAppMinutesPlayedData(betterproto.Message):
    minutes_played: List["CMsgClientAppMinutesPlayedDataAppMinutesPlayedData"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientAppMinutesPlayedDataAppMinutesPlayedData(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    forever: int = betterproto.int32_field(2)
    last_two_weeks: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientIsLimitedAccount(betterproto.Message):
    bis_limited_account: bool = betterproto.bool_field(1)
    bis_community_banned: bool = betterproto.bool_field(2)
    bis_locked_account: bool = betterproto.bool_field(3)
    bis_limited_account_allowed_to_invite_friends: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientRequestedClientStats(betterproto.Message):
    stats_to_send: List["CMsgClientRequestedClientStatsStatsToSend"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientRequestedClientStatsStatsToSend(betterproto.Message):
    client_stat: int = betterproto.uint32_field(1)
    stat_aggregate_method: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientStat2(betterproto.Message):
    stat_detail: List["CMsgClientStat2StatDetail"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientStat2StatDetail(betterproto.Message):
    client_stat: int = betterproto.uint32_field(1)
    ll_value: int = betterproto.int64_field(2)
    time_of_day: int = betterproto.uint32_field(3)
    cell_id: int = betterproto.uint32_field(4)
    depot_id: int = betterproto.uint32_field(5)
    app_id: int = betterproto.uint32_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientMMSSetRatelimitPolicyOnClient(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    enable_rate_limits: bool = betterproto.bool_field(2)
    seconds_per_message: int = betterproto.int32_field(3)
    milliseconds_per_data_update: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientMMSCreateLobby(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    max_members: int = betterproto.int32_field(2)
    lobby_type: int = betterproto.int32_field(3)
    lobby_flags: int = betterproto.int32_field(4)
    cell_id: int = betterproto.uint32_field(5)
    deprecated_public_ip: int = betterproto.uint32_field(6)
    metadata: bytes = betterproto.bytes_field(7)
    persona_name_owner: str = betterproto.string_field(8)
    public_ip: "CMsgIpAddress" = betterproto.message_field(9)


@dataclass(eq=False, repr=False)
class CMsgClientMMSCreateLobbyResponse(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    eresult: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientMMSJoinLobby(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    persona_name: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientMMSJoinLobbyResponse(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    chat_room_enter_response: int = betterproto.int32_field(3)
    max_members: int = betterproto.int32_field(4)
    lobby_type: int = betterproto.int32_field(5)
    lobby_flags: int = betterproto.int32_field(6)
    steam_id_owner: float = betterproto.fixed64_field(7)
    metadata: bytes = betterproto.bytes_field(8)
    members: List["CMsgClientMMSJoinLobbyResponseMember"] = betterproto.message_field(9)


@dataclass(eq=False, repr=False)
class CMsgClientMMSJoinLobbyResponseMember(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)
    persona_name: str = betterproto.string_field(2)
    metadata: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientMMSLeaveLobby(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientMMSLeaveLobbyResponse(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    eresult: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientMMSGetLobbyList(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    num_lobbies_requested: int = betterproto.int32_field(3)
    cell_id: int = betterproto.uint32_field(4)
    deprecated_public_ip: int = betterproto.uint32_field(5)
    filters: List["CMsgClientMMSGetLobbyListFilter"] = betterproto.message_field(6)
    public_ip: "CMsgIpAddress" = betterproto.message_field(7)


@dataclass(eq=False, repr=False)
class CMsgClientMMSGetLobbyListFilter(betterproto.Message):
    key: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)
    comparision: int = betterproto.int32_field(3)
    filter_type: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientMMSGetLobbyListResponse(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    eresult: int = betterproto.int32_field(3)
    lobbies: List["CMsgClientMMSGetLobbyListResponseLobby"] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientMMSGetLobbyListResponseLobby(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)
    max_members: int = betterproto.int32_field(2)
    lobby_type: int = betterproto.int32_field(3)
    lobby_flags: int = betterproto.int32_field(4)
    metadata: bytes = betterproto.bytes_field(5)
    num_members: int = betterproto.int32_field(6)
    distance: float = betterproto.float_field(7)
    weight: int = betterproto.int64_field(8)


@dataclass(eq=False, repr=False)
class CMsgClientMMSSetLobbyData(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    steam_id_member: float = betterproto.fixed64_field(3)
    max_members: int = betterproto.int32_field(4)
    lobby_type: int = betterproto.int32_field(5)
    lobby_flags: int = betterproto.int32_field(6)
    metadata: bytes = betterproto.bytes_field(7)


@dataclass(eq=False, repr=False)
class CMsgClientMMSSetLobbyDataResponse(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    eresult: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientMMSGetLobbyData(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientMMSLobbyData(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    num_members: int = betterproto.int32_field(3)
    max_members: int = betterproto.int32_field(4)
    lobby_type: int = betterproto.int32_field(5)
    lobby_flags: int = betterproto.int32_field(6)
    steam_id_owner: float = betterproto.fixed64_field(7)
    metadata: bytes = betterproto.bytes_field(8)
    members: List["CMsgClientMMSLobbyDataMember"] = betterproto.message_field(9)
    lobby_cellid: int = betterproto.uint32_field(10)
    owner_should_accept_changes: bool = betterproto.bool_field(11)


@dataclass(eq=False, repr=False)
class CMsgClientMMSLobbyDataMember(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)
    persona_name: str = betterproto.string_field(2)
    metadata: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientMMSSendLobbyChatMsg(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    steam_id_target: float = betterproto.fixed64_field(3)
    lobby_message: bytes = betterproto.bytes_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientMMSLobbyChatMsg(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    steam_id_sender: float = betterproto.fixed64_field(3)
    lobby_message: bytes = betterproto.bytes_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientMMSSetLobbyOwner(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    steam_id_new_owner: float = betterproto.fixed64_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientMMSSetLobbyOwnerResponse(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    eresult: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientMMSSetLobbyLinked(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    steam_id_lobby2: float = betterproto.fixed64_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientMMSSetLobbyGameServer(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    deprecated_game_server_ip: int = betterproto.uint32_field(3)
    game_server_port: int = betterproto.uint32_field(4)
    game_server_steam_id: float = betterproto.fixed64_field(5)
    game_server_ip: "CMsgIpAddress" = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientMMSLobbyGameServerSet(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    deprecated_game_server_ip: int = betterproto.uint32_field(3)
    game_server_port: int = betterproto.uint32_field(4)
    game_server_steam_id: float = betterproto.fixed64_field(5)
    game_server_ip: "CMsgIpAddress" = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientMMSUserJoinedLobby(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    steam_id_user: float = betterproto.fixed64_field(3)
    persona_name: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientMMSUserLeftLobby(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    steam_id_user: float = betterproto.fixed64_field(3)
    persona_name: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientMMSInviteToLobby(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    steam_id_user_invited: float = betterproto.fixed64_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientMMSGetLobbyStatus(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    claim_membership: bool = betterproto.bool_field(3)
    claim_ownership: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientMMSGetLobbyStatusResponse(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steam_id_lobby: float = betterproto.fixed64_field(2)
    lobby_status: "EMMSLobbyStatus" = betterproto.enum_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientInviteToGame(betterproto.Message):
    steam_id_dest: float = betterproto.fixed64_field(1)
    steam_id_src: float = betterproto.fixed64_field(2)
    connect_string: str = betterproto.string_field(3)
    remote_play: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientChatInvite(betterproto.Message):
    steam_id_invited: float = betterproto.fixed64_field(1)
    steam_id_chat: float = betterproto.fixed64_field(2)
    steam_id_patron: float = betterproto.fixed64_field(3)
    chatroom_type: int = betterproto.int32_field(4)
    steam_id_friend_chat: float = betterproto.fixed64_field(5)
    chat_name: str = betterproto.string_field(6)
    game_id: float = betterproto.fixed64_field(7)


@dataclass(eq=False, repr=False)
class CMsgClientConnectionStats(betterproto.Message):
    stats_logon: "CMsgClientConnectionStatsStats_Logon" = betterproto.message_field(1)
    stats_vconn: "CMsgClientConnectionStatsStats_VConn" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientConnectionStatsStats_Logon(betterproto.Message):
    connect_attempts: int = betterproto.int32_field(1)
    connect_successes: int = betterproto.int32_field(2)
    connect_failures: int = betterproto.int32_field(3)
    connections_dropped: int = betterproto.int32_field(4)
    seconds_running: int = betterproto.uint32_field(5)
    msec_tologonthistime: int = betterproto.uint32_field(6)
    count_bad_cms: int = betterproto.uint32_field(7)


@dataclass(eq=False, repr=False)
class CMsgClientConnectionStatsStats_UDP(betterproto.Message):
    pkts_sent: int = betterproto.uint64_field(1)
    bytes_sent: int = betterproto.uint64_field(2)
    pkts_recv: int = betterproto.uint64_field(3)
    pkts_processed: int = betterproto.uint64_field(4)
    bytes_recv: int = betterproto.uint64_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientConnectionStatsStats_VConn(betterproto.Message):
    connections_udp: int = betterproto.uint32_field(1)
    connections_tcp: int = betterproto.uint32_field(2)
    stats_udp: "CMsgClientConnectionStatsStats_UDP" = betterproto.message_field(3)
    pkts_abandoned: int = betterproto.uint64_field(4)
    conn_req_received: int = betterproto.uint64_field(5)
    pkts_resent: int = betterproto.uint64_field(6)
    msgs_sent: int = betterproto.uint64_field(7)
    msgs_sent_failed: int = betterproto.uint64_field(8)
    msgs_recv: int = betterproto.uint64_field(9)
    datagrams_sent: int = betterproto.uint64_field(10)
    datagrams_recv: int = betterproto.uint64_field(11)
    bad_pkts_recv: int = betterproto.uint64_field(12)
    unknown_conn_pkts_recv: int = betterproto.uint64_field(13)
    missed_pkts_recv: int = betterproto.uint64_field(14)
    dup_pkts_recv: int = betterproto.uint64_field(15)
    failed_connect_challenges: int = betterproto.uint64_field(16)
    micro_sec_avg_latency: int = betterproto.uint32_field(17)
    micro_sec_min_latency: int = betterproto.uint32_field(18)
    micro_sec_max_latency: int = betterproto.uint32_field(19)
    mem_pool_msg_in_use: int = betterproto.uint32_field(20)


@dataclass(eq=False, repr=False)
class CMsgClientServersAvailable(betterproto.Message):
    server_types_available: List["CMsgClientServersAvailableServer_Types_Available"] = betterproto.message_field(1)
    server_type_for_auth_services: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientServersAvailableServer_Types_Available(betterproto.Message):
    server: int = betterproto.uint32_field(1)
    changed: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientGetUserStats(betterproto.Message):
    game_id: float = betterproto.fixed64_field(1)
    crc_stats: int = betterproto.uint32_field(2)
    schema_local_version: int = betterproto.int32_field(3)
    steam_id_for_user: float = betterproto.fixed64_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientGetUserStatsResponse(betterproto.Message):
    game_id: float = betterproto.fixed64_field(1)
    eresult: int = betterproto.int32_field(2)
    crc_stats: int = betterproto.uint32_field(3)
    schema: bytes = betterproto.bytes_field(4)
    stats: List["CMsgClientGetUserStatsResponseStats"] = betterproto.message_field(5)
    achievement_blocks: List["CMsgClientGetUserStatsResponseAchievement_Blocks"] = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientGetUserStatsResponseStats(betterproto.Message):
    stat_id: int = betterproto.uint32_field(1)
    stat_value: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientGetUserStatsResponseAchievement_Blocks(betterproto.Message):
    achievement_id: int = betterproto.uint32_field(1)
    unlock_time: List[float] = betterproto.fixed32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientStoreUserStatsResponse(betterproto.Message):
    game_id: float = betterproto.fixed64_field(1)
    eresult: int = betterproto.int32_field(2)
    crc_stats: int = betterproto.uint32_field(3)
    stats_failed_validation: List[
        "CMsgClientStoreUserStatsResponseStats_Failed_Validation"
    ] = betterproto.message_field(4)
    stats_out_of_date: bool = betterproto.bool_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientStoreUserStatsResponseStats_Failed_Validation(betterproto.Message):
    stat_id: int = betterproto.uint32_field(1)
    reverted_stat_value: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientStoreUserStats2(betterproto.Message):
    game_id: float = betterproto.fixed64_field(1)
    settor_steam_id: float = betterproto.fixed64_field(2)
    settee_steam_id: float = betterproto.fixed64_field(3)
    crc_stats: int = betterproto.uint32_field(4)
    explicit_reset: bool = betterproto.bool_field(5)
    stats: List["CMsgClientStoreUserStats2Stats"] = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientStoreUserStats2Stats(betterproto.Message):
    stat_id: int = betterproto.uint32_field(1)
    stat_value: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientStatsUpdated(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)
    game_id: float = betterproto.fixed64_field(2)
    crc_stats: int = betterproto.uint32_field(3)
    updated_stats: List["CMsgClientStatsUpdatedUpdated_Stats"] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientStatsUpdatedUpdated_Stats(betterproto.Message):
    stat_id: int = betterproto.uint32_field(1)
    stat_value: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientStoreUserStats(betterproto.Message):
    game_id: float = betterproto.fixed64_field(1)
    explicit_reset: bool = betterproto.bool_field(2)
    stats_to_store: List["CMsgClientStoreUserStatsStats_To_Store"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientStoreUserStatsStats_To_Store(betterproto.Message):
    stat_id: int = betterproto.uint32_field(1)
    stat_value: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientGetClientDetails(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CMsgClientReportOverlayDetourFailure(betterproto.Message):
    failure_strings: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientGetClientDetailsResponse(betterproto.Message):
    package_version: int = betterproto.uint32_field(1)
    protocol_version: int = betterproto.uint32_field(8)
    os: str = betterproto.string_field(2)
    machine_name: str = betterproto.string_field(3)
    ip_public: str = betterproto.string_field(4)
    ip_private: str = betterproto.string_field(5)
    bytes_available: int = betterproto.uint64_field(7)
    games_running: List["CMsgClientGetClientDetailsResponseGame"] = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientGetClientDetailsResponseGame(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    extra_info: str = betterproto.string_field(2)
    time_running_sec: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientGetClientAppList(betterproto.Message):
    media: bool = betterproto.bool_field(1)
    tools: bool = betterproto.bool_field(2)
    games: bool = betterproto.bool_field(3)
    only_installed: bool = betterproto.bool_field(4)
    only_changing: bool = betterproto.bool_field(5)
    comics: bool = betterproto.bool_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientGetClientAppListResponse(betterproto.Message):
    apps: List["CMsgClientGetClientAppListResponseApp"] = betterproto.message_field(1)
    bytes_available: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientGetClientAppListResponseApp(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    category: str = betterproto.string_field(2)
    app_type: str = betterproto.string_field(10)
    favorite: bool = betterproto.bool_field(3)
    installed: bool = betterproto.bool_field(4)
    auto_update: bool = betterproto.bool_field(5)
    bytes_downloaded: int = betterproto.uint64_field(6)
    bytes_needed: int = betterproto.uint64_field(7)
    bytes_download_rate: int = betterproto.uint32_field(8)
    download_paused: bool = betterproto.bool_field(11)
    num_downloading: int = betterproto.uint32_field(12)
    num_paused: int = betterproto.uint32_field(13)
    changing: bool = betterproto.bool_field(14)
    available_on_platform: bool = betterproto.bool_field(15)
    dlcs: List["CMsgClientGetClientAppListResponseAppDLC"] = betterproto.message_field(9)


@dataclass(eq=False, repr=False)
class CMsgClientGetClientAppListResponseAppDLC(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    installed: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientInstallClientApp(betterproto.Message):
    appid: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientInstallClientAppResponse(betterproto.Message):
    result: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientUninstallClientApp(betterproto.Message):
    appid: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientUninstallClientAppResponse(betterproto.Message):
    result: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientSetClientAppUpdateState(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    update: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientSetClientAppUpdateStateResponse(betterproto.Message):
    result: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientUFSUploadFileRequest(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    file_size: int = betterproto.uint32_field(2)
    raw_file_size: int = betterproto.uint32_field(3)
    sha_file: bytes = betterproto.bytes_field(4)
    time_stamp: int = betterproto.uint64_field(5)
    file_name: str = betterproto.string_field(6)
    platforms_to_sync_deprecated: int = betterproto.uint32_field(7)
    platforms_to_sync: int = betterproto.uint32_field(8)
    cell_id: int = betterproto.uint32_field(9)
    can_encrypt: bool = betterproto.bool_field(10)


@dataclass(eq=False, repr=False)
class CMsgClientUFSUploadFileResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    sha_file: bytes = betterproto.bytes_field(2)
    use_http: bool = betterproto.bool_field(3)
    http_host: str = betterproto.string_field(4)
    http_url: str = betterproto.string_field(5)
    kv_headers: bytes = betterproto.bytes_field(6)
    use_https: bool = betterproto.bool_field(7)
    encrypt_file: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class CMsgClientUFSUploadCommit(betterproto.Message):
    files: List["CMsgClientUFSUploadCommitFile"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientUFSUploadCommitFile(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    app_id: int = betterproto.uint32_field(2)
    sha_file: bytes = betterproto.bytes_field(3)
    cub_file: int = betterproto.uint32_field(4)
    file_name: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientUFSUploadCommitResponse(betterproto.Message):
    files: List["CMsgClientUFSUploadCommitResponseFile"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientUFSUploadCommitResponseFile(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    app_id: int = betterproto.uint32_field(2)
    sha_file: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientUFSFileChunk(betterproto.Message):
    sha_file: bytes = betterproto.bytes_field(1)
    file_start: int = betterproto.uint32_field(2)
    data: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientUFSTransferHeartbeat(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CMsgClientUFSUploadFileFinished(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    sha_file: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUFSDeleteFileRequest(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    file_name: str = betterproto.string_field(2)
    is_explicit_delete: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientUFSDeleteFileResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    file_name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUFSGetFileListForApp(betterproto.Message):
    apps_to_query: List[int] = betterproto.uint32_field(1)
    send_path_prefixes: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUFSGetFileListForAppResponse(betterproto.Message):
    files: List["CMsgClientUFSGetFileListForAppResponseFile"] = betterproto.message_field(1)
    path_prefixes: List[str] = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUFSGetFileListForAppResponseFile(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    file_name: str = betterproto.string_field(2)
    sha_file: bytes = betterproto.bytes_field(3)
    time_stamp: int = betterproto.uint64_field(4)
    raw_file_size: int = betterproto.uint32_field(5)
    is_explicit_delete: bool = betterproto.bool_field(6)
    platforms_to_sync: int = betterproto.uint32_field(7)
    path_prefix_index: int = betterproto.uint32_field(8)


@dataclass(eq=False, repr=False)
class CMsgClientUFSDownloadRequest(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    file_name: str = betterproto.string_field(2)
    can_handle_http: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientUFSDownloadResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    app_id: int = betterproto.uint32_field(2)
    file_size: int = betterproto.uint32_field(3)
    raw_file_size: int = betterproto.uint32_field(4)
    sha_file: bytes = betterproto.bytes_field(5)
    time_stamp: int = betterproto.uint64_field(6)
    is_explicit_delete: bool = betterproto.bool_field(7)
    use_http: bool = betterproto.bool_field(8)
    http_host: str = betterproto.string_field(9)
    http_url: str = betterproto.string_field(10)
    kv_headers: bytes = betterproto.bytes_field(11)
    use_https: bool = betterproto.bool_field(12)
    encrypted: bool = betterproto.bool_field(13)


@dataclass(eq=False, repr=False)
class CMsgClientUFSLoginRequest(betterproto.Message):
    protocol_version: int = betterproto.uint32_field(1)
    am_session_token: int = betterproto.uint64_field(2)
    apps: List[int] = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientUFSLoginResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientWalletInfoUpdate(betterproto.Message):
    has_wallet: bool = betterproto.bool_field(1)
    balance: int = betterproto.int32_field(2)
    currency: int = betterproto.int32_field(3)
    balance_delayed: int = betterproto.int32_field(4)
    balance64: int = betterproto.int64_field(5)
    balance64_delayed: int = betterproto.int64_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientAppInfoUpdate(betterproto.Message):
    last_changenumber: int = betterproto.uint32_field(1)
    send_changelist: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientAppInfoChanges(betterproto.Message):
    current_change_number: int = betterproto.uint32_field(1)
    force_full_update: bool = betterproto.bool_field(2)
    app_ids: List[int] = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientAppInfoRequest(betterproto.Message):
    apps: List["CMsgClientAppInfoRequestApp"] = betterproto.message_field(1)
    supports_batches: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientAppInfoRequestApp(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    section_flags: int = betterproto.uint32_field(2)
    section_crc: List[int] = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientAppInfoResponse(betterproto.Message):
    apps: List["CMsgClientAppInfoResponseApp"] = betterproto.message_field(1)
    apps_unknown: List[int] = betterproto.uint32_field(2)
    apps_pending: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientAppInfoResponseApp(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    change_number: int = betterproto.uint32_field(2)
    sections: List["CMsgClientAppInfoResponseAppSection"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientAppInfoResponseAppSection(betterproto.Message):
    section_id: int = betterproto.uint32_field(1)
    section_kv: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientPackageInfoRequest(betterproto.Message):
    package_ids: List[int] = betterproto.uint32_field(1)
    meta_data_only: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientPackageInfoResponse(betterproto.Message):
    packages: List["CMsgClientPackageInfoResponsePackage"] = betterproto.message_field(1)
    packages_unknown: List[int] = betterproto.uint32_field(2)
    packages_pending: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientPackageInfoResponsePackage(betterproto.Message):
    package_id: int = betterproto.uint32_field(1)
    change_number: int = betterproto.uint32_field(2)
    sha: bytes = betterproto.bytes_field(3)
    buffer: bytes = betterproto.bytes_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientPICSChangesSinceRequest(betterproto.Message):
    since_change_number: int = betterproto.uint32_field(1)
    send_app_info_changes: bool = betterproto.bool_field(2)
    send_package_info_changes: bool = betterproto.bool_field(3)
    num_app_info_cached: int = betterproto.uint32_field(4)
    num_package_info_cached: int = betterproto.uint32_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientPICSChangesSinceResponse(betterproto.Message):
    current_change_number: int = betterproto.uint32_field(1)
    since_change_number: int = betterproto.uint32_field(2)
    force_full_update: bool = betterproto.bool_field(3)
    package_changes: List["CMsgClientPICSChangesSinceResponsePackageChange"] = betterproto.message_field(4)
    app_changes: List["CMsgClientPICSChangesSinceResponseAppChange"] = betterproto.message_field(5)
    force_full_app_update: bool = betterproto.bool_field(6)
    force_full_package_update: bool = betterproto.bool_field(7)


@dataclass(eq=False, repr=False)
class CMsgClientPICSChangesSinceResponsePackageChange(betterproto.Message):
    packageid: int = betterproto.uint32_field(1)
    change_number: int = betterproto.uint32_field(2)
    needs_token: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientPICSChangesSinceResponseAppChange(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    change_number: int = betterproto.uint32_field(2)
    needs_token: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientPICSProductInfoRequest(betterproto.Message):
    packages: List["CMsgClientPICSProductInfoRequestPackageInfo"] = betterproto.message_field(1)
    apps: List["CMsgClientPICSProductInfoRequestAppInfo"] = betterproto.message_field(2)
    meta_data_only: bool = betterproto.bool_field(3)
    num_prev_failed: int = betterproto.uint32_field(4)
    supports_package_tokens: int = betterproto.uint32_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientPICSProductInfoRequestAppInfo(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    access_token: int = betterproto.uint64_field(2)
    only_public: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientPICSProductInfoRequestPackageInfo(betterproto.Message):
    packageid: int = betterproto.uint32_field(1)
    access_token: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientPICSProductInfoResponse(betterproto.Message):
    apps: List["CMsgClientPICSProductInfoResponseAppInfo"] = betterproto.message_field(1)
    unknown_appids: List[int] = betterproto.uint32_field(2)
    packages: List["CMsgClientPICSProductInfoResponsePackageInfo"] = betterproto.message_field(3)
    unknown_packageids: List[int] = betterproto.uint32_field(4)
    meta_data_only: bool = betterproto.bool_field(5)
    response_pending: bool = betterproto.bool_field(6)
    http_min_size: int = betterproto.uint32_field(7)
    http_host: str = betterproto.string_field(8)


@dataclass(eq=False, repr=False)
class CMsgClientPICSProductInfoResponseAppInfo(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    change_number: int = betterproto.uint32_field(2)
    missing_token: bool = betterproto.bool_field(3)
    sha: bytes = betterproto.bytes_field(4)
    buffer: bytes = betterproto.bytes_field(5)
    only_public: bool = betterproto.bool_field(6)
    size: int = betterproto.uint32_field(7)


@dataclass(eq=False, repr=False)
class CMsgClientPICSProductInfoResponsePackageInfo(betterproto.Message):
    packageid: int = betterproto.uint32_field(1)
    change_number: int = betterproto.uint32_field(2)
    missing_token: bool = betterproto.bool_field(3)
    sha: bytes = betterproto.bytes_field(4)
    buffer: bytes = betterproto.bytes_field(5)
    size: int = betterproto.uint32_field(6)


@dataclass(eq=False, repr=False)
class CMsgClientPICSAccessTokenRequest(betterproto.Message):
    packageids: List[int] = betterproto.uint32_field(1)
    appids: List[int] = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientPICSAccessTokenResponse(betterproto.Message):
    package_access_tokens: List["CMsgClientPICSAccessTokenResponsePackageToken"] = betterproto.message_field(1)
    package_denied_tokens: List[int] = betterproto.uint32_field(2)
    app_access_tokens: List["CMsgClientPICSAccessTokenResponseAppToken"] = betterproto.message_field(3)
    app_denied_tokens: List[int] = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientPICSAccessTokenResponsePackageToken(betterproto.Message):
    packageid: int = betterproto.uint32_field(1)
    access_token: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientPICSAccessTokenResponseAppToken(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    access_token: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUFSGetUGCDetails(betterproto.Message):
    hcontent: float = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientUFSGetUGCDetailsResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    url: str = betterproto.string_field(2)
    app_id: int = betterproto.uint32_field(3)
    filename: str = betterproto.string_field(4)
    steamid_creator: float = betterproto.fixed64_field(5)
    file_size: int = betterproto.uint32_field(6)
    compressed_file_size: int = betterproto.uint32_field(7)
    rangecheck_host: str = betterproto.string_field(8)
    file_encoded_sha1: str = betterproto.string_field(9)


@dataclass(eq=False, repr=False)
class CMsgClientUFSGetSingleFileInfo(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    file_name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUFSGetSingleFileInfoResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    app_id: int = betterproto.uint32_field(2)
    file_name: str = betterproto.string_field(3)
    sha_file: bytes = betterproto.bytes_field(4)
    time_stamp: int = betterproto.uint64_field(5)
    raw_file_size: int = betterproto.uint32_field(6)
    is_explicit_delete: bool = betterproto.bool_field(7)


@dataclass(eq=False, repr=False)
class CMsgClientUFSShareFile(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    file_name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUFSShareFileResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    hcontent: float = betterproto.fixed64_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientAMGetClanOfficers(betterproto.Message):
    steamid_clan: float = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientAMGetClanOfficersResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    steamid_clan: float = betterproto.fixed64_field(2)
    officer_count: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientAMGetPersonaNameHistory(betterproto.Message):
    id_count: int = betterproto.int32_field(1)
    ids: List["CMsgClientAMGetPersonaNameHistoryIdInstance"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientAMGetPersonaNameHistoryIdInstance(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientAMGetPersonaNameHistoryResponse(betterproto.Message):
    responses: List["CMsgClientAMGetPersonaNameHistoryResponseNameTableInstance"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientAMGetPersonaNameHistoryResponseNameTableInstance(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    steamid: float = betterproto.fixed64_field(2)
    names: List["CMsgClientAMGetPersonaNameHistoryResponseNameTableInstanceNameInstance"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientAMGetPersonaNameHistoryResponseNameTableInstanceNameInstance(betterproto.Message):
    name_since: float = betterproto.fixed32_field(1)
    name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientDeregisterWithServer(betterproto.Message):
    eservertype: int = betterproto.uint32_field(1)
    app_id: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientClanState(betterproto.Message):
    steamid_clan: float = betterproto.fixed64_field(1)
    clan_account_flags: int = betterproto.uint32_field(3)
    name_info: "CMsgClientClanStateNameInfo" = betterproto.message_field(4)
    user_counts: "CMsgClientClanStateUserCounts" = betterproto.message_field(5)
    events: List["CMsgClientClanStateEvent"] = betterproto.message_field(6)
    announcements: List["CMsgClientClanStateEvent"] = betterproto.message_field(7)
    chat_room_private: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class CMsgClientClanStateNameInfo(betterproto.Message):
    clan_name: str = betterproto.string_field(1)
    sha_avatar: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientClanStateUserCounts(betterproto.Message):
    members: int = betterproto.uint32_field(1)
    online: int = betterproto.uint32_field(2)
    chatting: int = betterproto.uint32_field(3)
    in_game: int = betterproto.uint32_field(4)
    chat_room_members: int = betterproto.uint32_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientClanStateEvent(betterproto.Message):
    gid: float = betterproto.fixed64_field(1)
    event_time: int = betterproto.uint32_field(2)
    headline: str = betterproto.string_field(3)
    game_id: float = betterproto.fixed64_field(4)
    just_posted: bool = betterproto.bool_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientUnsignedInstallScript(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    file_name: str = betterproto.string_field(2)
    file_size: int = betterproto.uint32_field(3)
    signature_broken: bool = betterproto.bool_field(4)
    depot_id: int = betterproto.uint32_field(5)
    manifest_id: int = betterproto.uint64_field(6)
    file_flags: int = betterproto.uint32_field(7)



# -*- coding: utf-8 -*-

"""
steam.py
~~~~~~~~~~~~~
A basic wrapper for the Steam API and its Community Managers.

:copyright: (c) 2020 James.
:license: MIT, see LICENSE for more details.
"""

__title__ = "steam"
__author__ = "Gobot1234"
__license__ = "MIT"
__version__ = "0.6.1"

import logging
from typing import NamedTuple

from typing_extensions import Literal

from . import abc, guard, utils
from .abc import *
from .badge import *
from .channel import *
from .clan import *
from .client import *
from .comment import *
from .enums import *
from .errors import *
from .game import *
from .group import *
from .image import *
from .invite import *
from .message import *
from .models import *
from .role import *
from .trade import *
from .user import *


class VersionInfo(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: Literal["alpha", "beta", "candidate", "final"]


version_info = VersionInfo(major=0, minor=6, micro=1, releaselevel="final")

logging.getLogger(__name__).addHandler(logging.NullHandler())

# not for export
del logging, NamedTuple, Literal, VersionInfo



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import abc
import sys
import traceback
from typing import TYPE_CHECKING, Optional

from typing_extensions import final

from .commands import Command, GroupCommand
from .context import Context

if TYPE_CHECKING:
    from steam.ext import commands

    from .cog import Cog

__all__ = (
    "HelpCommand",
    "DefaultHelpCommand",
)


class HelpCommand(Command):
    """The base implementation of the help command.

    Attributes
    ----------
    context: :class:`~steam.ext.commands.Context`
        The context for the command's invocation.
    """

    context: Context

    def __init__(self, **kwargs):
        default = dict(name="help", help="Shows this message.", cog=self)
        default.update(kwargs)
        super().__init__(self.command_callback, **default)

    @final
    async def command_callback(self, ctx: Context, *, content: str = None) -> None:
        """|coro|
        The actual implementation of the help command.

        This method should not directly subclassed instead you should change the behaviour through the methods that
        actually get dispatched:

            - :meth:`send_cog_help`
            - :meth:`send_command_help`
            - :meth:`send_group_help`
            - :meth:`command_not_found`
        """
        self.context = ctx
        bot = ctx.bot
        if content is None:
            mapping = self.get_bot_mapping()
            return await self.send_help(mapping)
        # check if it's a cog
        cog = bot.get_cog(content)
        if cog is not None:
            return await self.send_cog_help(cog)
        command = bot.get_command(content)
        if command is not None:
            return await (
                self.send_group_help(command) if isinstance(command, GroupCommand) else self.send_command_help(command)
            )

        await self.command_not_found(content)

    def get_bot_mapping(self) -> "dict[Optional[str], list[commands.Command]]":
        """
        Generate a mapping of the bot's commands. It's not normally necessary to subclass this.

        Returns
        -------
        dict[Optional[:class:`str`], list[:class:`Command`]]
            The mapping of commands passed to :meth:`send_help`.
        """
        bot = self.context.bot
        mapping = {cog.qualified_name: list(cog.commands) for name, cog in bot.cogs.items() if cog.commands}
        categorized_commands = [command for c in mapping.values() for command in c]
        mapping[None] = [c for c in bot.commands if c not in categorized_commands]
        return mapping

    @abc.abstractmethod
    async def send_help(self, mapping: "dict[Optional[str], list[commands.Command]]") -> None:
        """|coro|
        Send the basic help message for the bot's command.

        Parameters
        ----------
        mapping: dict[Optional[commands.Cog], list[commands.Command]]
            The mapping from :meth:`get_bot_mapping`.
        """

    @abc.abstractmethod
    async def send_cog_help(self, cog: "commands.Cog") -> None:
        """|coro|
        The method called with a cog is passed as an argument.

        Note
        ----
        Cogs are case-sensitive.


        Parameters
        ----------
        cog: :class:`~steam.ext.commands.Cog`
            The cog that was passed as an argument.
        """

    @abc.abstractmethod
    async def send_command_help(self, command: "commands.Command") -> None:
        """|coro|
        The method called when a normal command is passed as an argument.

        Parameters
        ----------
        command: :class:`~steam.ext.commands.Command`
            The command that was passed as an argument.
        """

    @abc.abstractmethod
    async def send_group_help(self, command: "commands.GroupCommand") -> None:
        """|coro|
        The method called when a group command is passed as an argument.

        Parameters
        ----------
        command: :class:`~steam.ext.commands.GroupCommand`
            The command that was passed as an argument.
        """

    @abc.abstractmethod
    async def command_not_found(self, command: str) -> None:
        """|coro|
        The default implementation for when a command isn't found.

        This by default sends "The command {command} was not found."

        Parameters
        ----------
        command: :class:`str`
            The command that was not found.
        """

    async def on_error(self, ctx: "commands.Context", error: Exception) -> None:
        """|coro|
        The default error handler for the help command. This performs the functionality as
        :meth:`steam.ext.commands.Bot.on_command_error`.

        Parameters
        ----------
        ctx: :class:`.Context`
            The context for the invocation.
        error: :exc:`Exception`
            The error that was raised.
        """
        print(f"Ignoring exception in command {ctx.command}:", file=sys.stderr)
        traceback.print_exception(type(error), error, error.__traceback__, file=sys.stderr)

    cog_command_error = on_error


class DefaultHelpCommand(HelpCommand):
    """The default implementation of the help command."""

    def __repr__(self) -> str:
        return "<default help command>"

    def _get_doc(self, command: Command) -> str:
        try:
            return command.help.splitlines()[0]
        except (IndexError, AttributeError):
            return ""

    async def send_help(self, mapping: "dict[Optional[str], list[commands.Command]]") -> None:
        message = ["/pre"]
        for cog_name, commands in mapping.items():
            (
                message.append(f"\n{cog_name}'s commands")
                if cog_name is not None
                else message.append("\nUn-categorized commands")
            )
            for command in commands:
                message.append(f'{command.name}{f": {self._get_doc(command)}" if command.help else ""}')
        await self.context.send("\n".join(message))

    async def send_cog_help(self, cog: "commands.Cog") -> None:
        message = [f"/pre {cog.qualified_name}'s commands"]
        for name in sorted(c.name for c in cog.commands):
            command = cog.__commands__[name]
            message.append(f'{name}{f": {self._get_doc(command)}" if command.help else ""}')
        await self.context.send("\n".join(message))

    async def send_command_help(self, command: "commands.Command") -> None:
        await self.context.send(f"/pre Help with {command.name}:\n\n{command.help}")

    async def send_group_help(self, command: "commands.GroupCommand") -> None:
        msg = [f"/pre Help with {command.name}:\n\n{command.help}"]
        sub_commands = "\n".join(c.name for c in command.children)
        if sub_commands:
            msg.append(f"\nAnd its sub commands:\n{sub_commands}")
        await self.context.send("\n".join(msg))

    async def command_not_found(self, command: str) -> None:
        """|coro|
        The default implementation for when a command isn't found.

        This by default sends "The command {command} was not found."

        Parameters
        ----------
        command: :class:`str`
            The command that was not found.
        """
        await self.context.send(f"The command {command!r} was not found.")



# -*- coding: utf-8 -*-

import platform

import pkg_resources


def get_version_info(package: str) -> str:
    return pkg_resources.get_distribution(package).version


def main() -> None:
    print("python version:", platform.python_version())
    print("steam.py version:", get_version_info("steam"))
    print("aiohttp version:", get_version_info("aiohttp"))
    print("betterproto version:", get_version_info("betterproto"))
    print("operating system info:", platform.platform())


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

Based on https://github.com/Rapptz/discord.py/blob/master/discord/file.py
"""

from __future__ import annotations

import hashlib
import imghdr
import io
import struct
from time import time
from typing import TYPE_CHECKING, Union

__all__ = ("Image",)

if TYPE_CHECKING:
    from _typeshed import AnyPath


class Image:
    """A wrapper around common image files. Used for :meth:`steam.User.send`.

    Parameters
    ----------
    fp: Union[:class:`io.FileIO`, :class:`str`]
        An image or path-like to pass to :func:`open`.
    spoiler: :class:`bool`
        Whether or not to mark the image as a spoiler.

    Note
    ----
    Currently supported image types include:
        - PNG
        - JPG/JPEG
        - GIF
    """

    # TODO add support for "webm", "mpg", "mp4", "mpeg", "ogv"

    __slots__ = ("fp", "spoiler", "name", "width", "height", "type", "hash")

    def __init__(self, fp: Union[io.BufferedIOBase, AnyPath], *, spoiler: bool = False):
        self.fp: io.BufferedIOBase = fp if isinstance(fp, io.BufferedIOBase) else open(fp, "rb")

        if not (self.fp.seekable() and self.fp.readable()):
            raise ValueError(f"File buffer {fp!r} must be seekable and readable")

        if len(self) > 10485760:
            raise ValueError("file is too large to upload")

        # from https://stackoverflow.com/questions/8032642
        headers = self.fp.read(24)
        if len(headers) != 24:
            raise ValueError("Opened file has no headers")
        self.type = imghdr.what(None, headers)
        if self.type == "png":
            check = struct.unpack(">i", headers[4:8])[0]
            if check != 0x0d0a1a0a:
                raise ValueError("Opened file's headers do not match a standard PNGs headers")
            width, height = struct.unpack(">ii", headers[16:24])
        elif self.type == "gif":
            width, height = struct.unpack("<HH", headers[6:10])
        elif self.type == "jpeg":
            try:
                self.fp.seek(0)  # read 0xff next
                size = 2
                ftype = 0
                while not 0xc0 <= ftype <= 0xcf or ftype in (0xc4, 0xc8, 0xcc):
                    self.fp.seek(size, 1)
                    byte = self.fp.read(1)
                    while ord(byte) == 0xff:
                        byte = self.fp.read(1)
                    ftype = ord(byte)
                    size = struct.unpack(">H", self.fp.read(2))[0] - 2
                # we are at a SOFn block
                self.fp.seek(1, 1)  # skip 'precision' byte.
                height, width = struct.unpack(">HH", self.fp.read(4))
            except Exception as exc:
                raise ValueError from exc
        else:
            raise TypeError("Unsupported file format passed")
        self.spoiler = spoiler
        self.width = width
        self.height = height
        self.hash = hashlib.sha1(self.read()).hexdigest()
        self.name = f'{int(time())}_{getattr(self.fp, "name", f"image.{self.type}")}'

    def __len__(self) -> int:
        return len(self.read())

    def read(self) -> bytes:
        self.fp.seek(0)
        read = self.fp.read()
        self.fp.seek(0)
        return read


def test_jpeg(h: bytes, _) -> str:  # adds support for more header types
    # SOI APP2 + ICC_PROFILE
    if h[0:4] == "\xff\xd8\xff\xe2" and h[6:17] == b"ICC_PROFILE":
        return "jpeg"
    # SOI APP14 + Adobe
    if h[0:4] == "\xff\xd8\xff\xee" and h[6:11] == b"Adobe":
        return "jpeg"
    # SOI DQT
    if h[0:4] == "\xff\xd8\xff\xdb":
        return "jpeg"


imghdr.tests.append(test_jpeg)



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import functools
from datetime import timedelta
from typing import Any, Callable, Optional, TypeVar, Union, overload

from typing_extensions import TypedDict

from .enums import IntEnum

__all__ = (
    "TF2",
    "Game",
    "CSGO",
    "DOTA2",
    "STEAM",
    "CUSTOM_GAME",
)

T = TypeVar("T")
APP_ID_MAX = 2 ** 32
GAMES: dict[int, list[Game]] = {}


class Games(IntEnum):
    Team_Fortress_2 = 440
    DOTA_2 = 570
    Counter_Strike_Global__Offensive = 730
    Steam = 753


class GameDict(TypedDict):
    name: str
    appid: str
    playtime_forever: int
    img_icon_url: str
    img_logo_url: str
    has_community_visible_stats: bool


class GameToDict(TypedDict, total=False):
    game_id: str
    game_extra_info: str


def cache_values(new: Callable[..., T]) -> Callable[..., T]:
    @functools.wraps(new)
    def inner(*args: Any, **kwargs: Any) -> T:
        game = new(*args, **kwargs)

        try:
            games = GAMES[game.id]
        except KeyError:
            GAMES[game.id] = games = [game]

        for game_ in games:
            if game_.title == game.title:
                return game_

        return game

    return inner


class Game:
    """Represents a Steam game.

    Note
    ----
    This class can be defined by users using the above parameters, or it can be from an API call this is when
    :meth:`~steam.User.fetch_games` is called.

    Parameters
    ----------
    title: Optional[:class:`str`]
        The game's title.
    id: Optional[:class:`int`]
        The game's app ID.
    context_id: Optional[:class:`int`]
        The game's context ID by default 2.

    Attributes
    -----------
    title: Optional[:class:`str`]
        The game's title.
    id: :class:`int`
        The game's app ID.
    context_id: :class:`int`
        The context id of the game normally ``2``.
    total_play_time: :class:`datetime.timedelta`
        The total time the game has been played for.
        Only applies to a :class:`~steam.User`'s games from :meth:`~steam.User.games`.
    icon_url: Optional[:class:`str`]
        The icon url of the game.
        Only applies to a :class:`~steam.User`'s games from :meth:`~steam.User.games`.
    logo_url: Optional[:class:`str`]
        The logo url of the game.
        Only applies to a :class:`~steam.User`'s games from :meth:`~steam.User.games`.
    """

    __slots__ = (
        "id",
        "title",
        "context_id",
        "total_play_time",
        "icon_url",
        "logo_url",
        "_stats_visible",
    )

    id: int
    title: Optional[str]
    context_id: Optional[int]

    total_play_time: Optional[timedelta]
    icon_url: Optional[str]
    logo_url: Optional[str]
    _stats_visible: Optional[bool]

    @overload
    def __new__(cls, *, id: Union[int, str], context_id: Optional[int] = None) -> Game:
        ...

    @overload
    def __new__(cls, *, title: str, context_id: Optional[int] = None) -> Game:
        ...

    @overload
    def __new__(cls, *, id: Union[int, str], title: str, context_id: Optional[int] = None) -> Game:
        ...

    @overload
    def __new__(
        cls, *, id: Optional[Union[int, str]] = None, title: Optional[str] = None, context_id: Optional[int] = None
    ) -> Game:
        ...

    @cache_values
    def __new__(
        cls, *, id: Optional[Union[int, str]] = None, title: Optional[str] = None, context_id: Optional[int] = None
    ) -> Game:
        if title is None and id is None:
            raise TypeError("__new__() missing a required keyword argument: 'id' or 'title'")

        self = super().__new__(cls)

        if title is None:
            try:
                id = int(id)
            except (ValueError, TypeError):
                raise ValueError(f"id expected to support int()")
            try:
                title = Games(id).name.replace("__", "-").replace("_", " ")
            except ValueError:
                title = None
            else:
                if title == "Steam" and context_id is None:
                    context_id = 6
        elif id is None:
            try:
                id = Games[title.replace(" ", "_").replace("-", "__")].value
            except KeyError:
                id = 0
        else:
            try:
                id = int(id)
            except (ValueError, TypeError):
                raise ValueError("id must be an int") from None

        if id < 0:
            raise ValueError("id cannot be negative")

        self.id = id
        self.title = title
        self.context_id = 2 if context_id is None else context_id

        self.total_play_time = None
        self.icon_url = None
        self.logo_url = None
        self._stats_visible = None

        return self

    @classmethod
    def _from_api(cls, data: GameDict) -> Game:
        game = cls(id=data.get("appid"), title=data.get("name"))
        game.total_play_time = timedelta(minutes=data.get("playtime_forever", 0))
        game.icon_url = (
            f"https://cdn.cloudflare.steamstatic.com/steamcommunity/public/images/apps/{game.id}/"
            f"{data.get('img_icon_url')}.jpg"
        )
        game.logo_url = (
            f"https://cdn.cloudflare.steamstatic.com/steamcommunity/public/images/apps/{game.id}"
            f"/{data.get('img_logo_url')}.jpg"
        )
        game._stats_visible = data.get("has_community_visible_stats", False)
        return game

    def __str__(self) -> str:
        return self.title or ""

    def __repr__(self) -> str:
        attrs = ("title", "id", "context_id")
        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
        return f"Game({', '.join(resolved)})"

    def to_dict(self) -> GameToDict:
        """dict[:class:`str`, :class:`str`]: The dict representation of the game used to set presences."""
        return (
            {"game_id": str(self.id)}
            if self.is_steam_game()
            else {"game_id": str(self.id), "game_extra_info": self.title}
        )

    def is_steam_game(self) -> bool:
        """:class:`bool`: Whether the game could be a Steam game."""
        return self.id <= APP_ID_MAX

    def has_visible_stats(self) -> bool:
        """:class:`bool`: Whether the game has publicly visible stats.
        Only applies to a :class:`~steam.User`'s games from :meth:`~steam.User.games`.
        """
        return self._stats_visible


TF2 = Game(title="Team Fortress 2")
DOTA2 = Game(title="DOTA 2")
CSGO = Game(title="Counter Strike Global-Offensive")
STEAM = Game(title="Steam", context_id=6)


def CUSTOM_GAME(title: str) -> Game:
    """Create a custom game instance for :meth:`~steam.Client.change_presence`.
    The :attr:`Game.id` will be set to ``15190414816125648896`` and the :attr:`Game.context_id` to ``None``.

    Example: ::

        await client.change_presence(game=steam.CUSTOM_GAME('my cool game'))

    Parameters
    ----------
    title: :class:`str`
        The name of the game to set your playing status to.

    Returns
    -------
    class:`Game`
        The created custom game.
    """
    return Game(title=title, id=15190414816125648896, context_id=None)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_inventory.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class CInventoryGetInventoryRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    steamid: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CInventoryResponse(betterproto.Message):
    etag: str = betterproto.string_field(1)
    removeditemids: List[int] = betterproto.uint64_field(2)
    item_json: str = betterproto.string_field(3)
    itemdef_json: str = betterproto.string_field(4)
    ticket: bytes = betterproto.bytes_field(5)
    replayed: bool = betterproto.bool_field(6)


@dataclass(eq=False, repr=False)
class CInventoryExchangeItemRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    steamid: int = betterproto.uint64_field(2)
    materialsitemid: List[int] = betterproto.uint64_field(3)
    materialsquantity: List[int] = betterproto.uint32_field(4)
    outputitemdefid: int = betterproto.uint64_field(5)


@dataclass(eq=False, repr=False)
class CInventoryGetEligiblePromoItemDefIDsRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    steamid: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CInventoryGetEligiblePromoItemDefIDsResponse(betterproto.Message):
    itemdefids: List[int] = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CInventoryAddItemRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    itemdefid: List[int] = betterproto.uint64_field(2)
    itempropsjson: List[str] = betterproto.string_field(3)
    steamid: int = betterproto.uint64_field(4)
    notify: bool = betterproto.bool_field(5)
    requestid: int = betterproto.uint64_field(6)
    trade_restriction: bool = betterproto.bool_field(7)
    is_purchase: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class CInventoryModifyItemsRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    steamid: int = betterproto.uint64_field(2)
    updates: List["CInventoryModifyItemsRequestItemPropertyUpdate"] = betterproto.message_field(3)
    timestamp: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class CInventoryModifyItemsRequestItemPropertyUpdate(betterproto.Message):
    itemid: int = betterproto.uint64_field(1)
    remove_property: bool = betterproto.bool_field(2)
    property_name: str = betterproto.string_field(3)
    property_value_bool: bool = betterproto.bool_field(4)
    property_value_int: int = betterproto.int64_field(5)
    property_value_string: str = betterproto.string_field(6)
    property_value_float: float = betterproto.float_field(7)


@dataclass(eq=False, repr=False)
class CInventoryConsumePlaytimeRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    itemdefid: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CInventoryConsumeItemRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    itemid: int = betterproto.uint64_field(2)
    quantity: int = betterproto.uint32_field(3)
    timestamp: str = betterproto.string_field(4)
    steamid: int = betterproto.uint64_field(5)
    requestid: int = betterproto.uint64_field(6)


@dataclass(eq=False, repr=False)
class CInventoryDevSetNextDropRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    itemdefid: int = betterproto.uint64_field(2)
    droptime: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CInventorySplitItemStackRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    itemid: int = betterproto.uint64_field(2)
    quantity: int = betterproto.uint32_field(3)
    steamid: int = betterproto.uint64_field(5)


@dataclass(eq=False, repr=False)
class CInventoryCombineItemStacksRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    fromitemid: int = betterproto.uint64_field(2)
    destitemid: int = betterproto.uint64_field(3)
    quantity: int = betterproto.uint32_field(4)
    steamid: int = betterproto.fixed64_field(7)


@dataclass(eq=False, repr=False)
class CInventoryGetItemDefMetaRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class CInventoryGetItemDefMetaResponse(betterproto.Message):
    modified: int = betterproto.uint32_field(1)
    digest: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CInventoryGetUserPurchaseInfoRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CInventoryGetUserPurchaseInfoResponse(betterproto.Message):
    ecurrency: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class CInventoryPurchaseInitRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    language: int = betterproto.int32_field(2)
    line_items: List["CInventoryPurchaseInitRequestLineItem"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CInventoryPurchaseInitRequestLineItem(betterproto.Message):
    itemdefid: int = betterproto.uint64_field(1)
    quantity: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CInventoryPurchaseInitResponse(betterproto.Message):
    orderid: int = betterproto.uint64_field(1)
    transid: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CInventoryPurchaseFinalizeRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    language: int = betterproto.int32_field(2)
    orderid: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class CInventoryInspectItemRequest(betterproto.Message):
    itemdefid: int = betterproto.uint64_field(1)
    itemid: int = betterproto.fixed64_field(2)
    tags: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CInventoryClientNewItemsNotification(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    inventory_response: "CInventoryResponse" = betterproto.message_field(2)



# -*- coding: utf-8 -*-

from . import (
    steammessages_base,
    steammessages_clientserver,
    steammessages_clientserver_2,
    steammessages_clientserver_friends,
    steammessages_clientserver_login,
)
from .emsg import *

__all__ = ("PROTOBUFS",)


PROTOBUFS = {
    EMsg.Multi: steammessages_base.CMsgMulti,
    EMsg.ClientHeartBeat: steammessages_clientserver_login.CMsgClientHeartBeat,
    EMsg.ClientLogOff: steammessages_clientserver_login.CMsgClientLogOff,
    EMsg.ClientConnectionStats: steammessages_clientserver.CMsgClientConnectionStats,
    EMsg.ClientRemoveFriend: steammessages_clientserver_friends.CMsgClientRemoveFriend,
    EMsg.ClientChangeStatus: steammessages_clientserver_friends.CMsgClientChangeStatus,
    EMsg.ClientFriendMsg: steammessages_clientserver_friends.CMsgClientFriendMsg,
    EMsg.ClientRedeemGuestPass: steammessages_clientserver_2.CMsgClientRedeemGuestPass,
    EMsg.ClientGamesPlayedWithDataBlob: steammessages_clientserver.CMsgClientGamesPlayed,
    EMsg.ClientRegisterKey: steammessages_clientserver_2.CMsgClientRegisterKey,
    EMsg.ClientPurchaseWithMachineID: steammessages_clientserver_2.CMsgClientPurchaseWithMachineID,
    EMsg.ClientLogOnResponse: steammessages_clientserver_login.CMsgClientLogonResponse,
    EMsg.ClientLoggedOff: steammessages_clientserver_login.CMsgClientLoggedOff,
    EMsg.ClientPurchaseResponse: steammessages_clientserver_2.CMsgClientPurchaseResponse,
    EMsg.ClientPersonaState: steammessages_clientserver_friends.CMsgClientPersonaState,
    EMsg.ClientFriendsList: steammessages_clientserver_friends.CMsgClientFriendsList,
    EMsg.ClientAccountInfo: steammessages_clientserver_login.CMsgClientAccountInfo,
    EMsg.GSStatusReply: steammessages_clientserver.CMsgGSStatusReply,
    EMsg.ClientGameConnectTokens: steammessages_clientserver.CMsgClientGameConnectTokens,
    EMsg.ClientLicenseList: steammessages_clientserver.CMsgClientLicenseList,
    EMsg.ClientCMList: steammessages_clientserver.CMsgClientCMList,
    EMsg.ClientAddFriend: steammessages_clientserver_friends.CMsgClientAddFriend,
    EMsg.ClientAddFriendResponse: steammessages_clientserver_friends.CMsgClientAddFriendResponse,
    EMsg.ClientChatInvite: steammessages_clientserver.CMsgClientChatInvite,
    EMsg.ClientRequestFriendData: steammessages_clientserver_friends.CMsgClientRequestFriendData,
    EMsg.ClientGetUserStats: steammessages_clientserver.CMsgClientGetUserStats,
    EMsg.ClientGetUserStatsResponse: steammessages_clientserver.CMsgClientGetUserStatsResponse,
    EMsg.ClientStoreUserStats: steammessages_clientserver.CMsgClientStoreUserStats,
    EMsg.ClientStoreUserStatsResponse: steammessages_clientserver.CMsgClientStoreUserStatsResponse,
    EMsg.ClientClanState: steammessages_clientserver.CMsgClientClanState,
    EMsg.ClientServiceModule: steammessages_clientserver_2.CMsgClientServiceModule,
    EMsg.ClientServiceCall: steammessages_clientserver_2.CMsgClientServiceCall,
    EMsg.ClientServiceCallResponse: steammessages_clientserver_2.CMsgClientServiceCallResponse,
    EMsg.ClientGetAppOwnershipTicket: steammessages_clientserver.CMsgClientGetAppOwnershipTicket,
    EMsg.ClientGetAppOwnershipTicketResponse: steammessages_clientserver.CMsgClientGetAppOwnershipTicketResponse,
    EMsg.ClientUpdateUserGameInfo: steammessages_clientserver_2.CMsgClientUpdateUserGameInfo,
    EMsg.ClientTicketAuthComplete: steammessages_clientserver.CMsgClientTicketAuthComplete,
    EMsg.ClientIsLimitedAccount: steammessages_clientserver.CMsgClientIsLimitedAccount,
    EMsg.ClientAuthList: steammessages_clientserver.CMsgClientAuthList,
    EMsg.ClientGetDepotDecryptionKey: steammessages_clientserver_2.CMsgClientGetDepotDecryptionKey,
    EMsg.ClientGetDepotDecryptionKeyResponse: steammessages_clientserver_2.CMsgClientGetDepotDecryptionKeyResponse,
    EMsg.ClientToGC: steammessages_clientserver_2.CMsgGcClient,
    EMsg.ClientFromGC: steammessages_clientserver_2.CMsgGcClient,
    EMsg.ClientEmailAddrInfo: steammessages_clientserver_2.CMsgClientEmailAddrInfo,
    EMsg.ClientNewLoginKey: steammessages_clientserver_login.CMsgClientNewLoginKey,
    EMsg.ClientNewLoginKeyAccepted: steammessages_clientserver_login.CMsgClientNewLoginKeyAccepted,
    EMsg.ClientStoreUserStats2: steammessages_clientserver.CMsgClientStoreUserStats2,
    EMsg.ClientStatsUpdated: steammessages_clientserver.CMsgClientStatsUpdated,
    EMsg.ClientLogon: steammessages_clientserver_login.CMsgClientLogon,
    EMsg.ClientGetClientDetails: steammessages_clientserver.CMsgClientGetClientDetails,
    EMsg.ClientGetClientDetailsResponse: steammessages_clientserver.CMsgClientGetClientDetailsResponse,
    EMsg.ClientReportOverlayDetourFailure: steammessages_clientserver.CMsgClientReportOverlayDetourFailure,
    EMsg.ClientGetClientAppList: steammessages_clientserver.CMsgClientGetClientAppList,
    EMsg.ClientGetClientAppListResponse: steammessages_clientserver.CMsgClientGetClientAppListResponse,
    EMsg.ClientInstallClientApp: steammessages_clientserver.CMsgClientInstallClientApp,
    EMsg.ClientInstallClientAppResponse: steammessages_clientserver.CMsgClientInstallClientAppResponse,
    EMsg.ClientUninstallClientApp: steammessages_clientserver.CMsgClientUninstallClientApp,
    EMsg.ClientUninstallClientAppResponse: steammessages_clientserver.CMsgClientUninstallClientAppResponse,
    EMsg.ClientSetClientAppUpdateState: steammessages_clientserver.CMsgClientSetClientAppUpdateState,
    EMsg.ClientSetClientAppUpdateStateResponse: steammessages_clientserver.CMsgClientSetClientAppUpdateStateResponse,
    EMsg.ClientWalletInfoUpdate: steammessages_clientserver.CMsgClientWalletInfoUpdate,
    EMsg.ClientFriendProfileInfo: steammessages_clientserver_friends.CMsgClientFriendProfileInfo,
    EMsg.ClientFriendProfileInfoResponse: steammessages_clientserver_friends.CMsgClientFriendProfileInfoResponse,
    EMsg.ClientGetCDNAuthToken: steammessages_clientserver_2.CMsgClientGetCdnAuthToken,
    EMsg.ClientGetCDNAuthTokenResponse: steammessages_clientserver_2.CMsgClientGetCdnAuthTokenResponse,
    EMsg.ClientRequestAccountData: steammessages_clientserver_2.CMsgClientRequestAccountData,
    EMsg.ClientRequestAccountDataResponse: steammessages_clientserver_2.CMsgClientRequestAccountDataResponse,
    EMsg.ClientHideFriend: steammessages_clientserver_friends.CMsgClientHideFriend,
    EMsg.ClientFriendsGroupsList: steammessages_clientserver_friends.CMsgClientFriendsGroupsList,
    EMsg.ClientGetClanActivityCounts: steammessages_clientserver_2.CMsgClientGetClanActivityCounts,
    EMsg.ClientGetClanActivityCountsResponse: steammessages_clientserver_2.CMsgClientGetClanActivityCountsResponse,
    EMsg.ClientLogonGameServer: steammessages_clientserver_login.CMsgClientLogon,
    EMsg.ClientItemAnnouncements: steammessages_clientserver_2.CMsgClientItemAnnouncements,
    EMsg.ClientRequestItemAnnouncements: steammessages_clientserver_2.CMsgClientRequestItemAnnouncements,
    EMsg.ClientCommentNotifications: steammessages_clientserver_2.CMsgClientCommentNotifications,
    EMsg.ClientRequestCommentNotifications: steammessages_clientserver_2.CMsgClientRequestCommentNotifications,
    EMsg.ClientFriendUserStatusPublished: steammessages_clientserver_2.CMsgClientFriendUserStatusPublished,
    EMsg.ClientCurrentUIMode: steammessages_clientserver_2.CMsgClientUiMode,
    EMsg.ClientVanityURLChangedNotification: steammessages_clientserver_2.CMsgClientVanityUrlChangedNotification,
    EMsg.ClientUserNotifications: steammessages_clientserver_2.CMsgClientUserNotifications,
    EMsg.BadgeCraftedNotification: steammessages_clientserver_2.CMsgBadgeCraftedNotification,
    EMsg.ClientNetworkingMobileCertRequest: steammessages_clientserver.CMsgClientNetworkingMobileCertRequest,
    EMsg.ClientGMSServerQuery: steammessages_clientserver_2.CMsgClientGmsServerQuery,
    EMsg.GMSClientServerQueryResponse: steammessages_clientserver_2.CMsgGmsClientServerQueryResponse,
    EMsg.GameServerOutOfDate: steammessages_clientserver_2.CMsgGameServerOutOfDate,
    EMsg.ClientInviteToGame: steammessages_clientserver.CMsgClientInviteToGame,
    EMsg.ClientRichPresenceUpload: steammessages_clientserver_2.CMsgClientRichPresenceUpload,
    EMsg.ClientRichPresenceRequest: steammessages_clientserver_2.CMsgClientRichPresenceRequest,
    EMsg.ClientRichPresenceInfo: steammessages_clientserver_2.CMsgClientRichPresenceInfo,
    EMsg.ClientChatOfflineMessageNotification: steammessages_clientserver_2.CMsgClientOfflineMessageNotification,
    EMsg.ClientChatGetFriendMessageHistory: steammessages_clientserver_2.CMsgClientChatGetFriendMessageHistory,
    EMsg.ClientChatGetFriendMessageHistoryResponse: steammessages_clientserver_2.CMsgClientChatGetFriendMessageHistoryResponse,
    EMsg.ClientChatGetFriendMessageHistoryForOfflineMessages: steammessages_clientserver_2.CMsgClientChatGetFriendMessageHistoryForOfflineMessages,
    EMsg.ClientFSGetFriendsSteamLevels: steammessages_clientserver_2.CMsgClientFsGetFriendsSteamLevels,
    EMsg.ClientFSGetFriendsSteamLevelsResponse: steammessages_clientserver_2.CMsgClientFsGetFriendsSteamLevelsResponse,
    EMsg.ClientPICSChangesSinceRequest: steammessages_clientserver.CMsgClientPICSChangesSinceRequest,
    EMsg.ClientPICSChangesSinceResponse: steammessages_clientserver.CMsgClientPICSChangesSinceResponse,
    EMsg.ClientPICSProductInfoRequest: steammessages_clientserver.CMsgClientPICSProductInfoRequest,
    EMsg.ClientPICSProductInfoResponse: steammessages_clientserver.CMsgClientPICSProductInfoResponse,
    EMsg.ClientPICSAccessTokenRequest: steammessages_clientserver.CMsgClientPICSAccessTokenRequest,
    EMsg.ClientPICSAccessTokenResponse: steammessages_clientserver.CMsgClientPICSAccessTokenResponse,
    EMsg.ClientGetEmoticonList: steammessages_clientserver_friends.CMsgClientGetEmoticonList,
    EMsg.ClientEmoticonList: steammessages_clientserver_friends.CMsgClientEmoticonList,
    EMsg.ClientPlayingSessionState: steammessages_clientserver_2.CMsgClientPlayingSessionState,
    EMsg.ClientKickPlayingSession: steammessages_clientserver_2.CMsgClientKickPlayingSession,
    EMsg.ClientServerTimestampRequest: steammessages_clientserver_login.CMsgClientServerTimestampRequest,
    EMsg.ClientServerTimestampResponse: steammessages_clientserver_login.CMsgClientServerTimestampResponse,
}
